/*
 *    Copyright 2006 Alejandro Gomez
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 *
 */
package ar.amgomez.idea.remotetail;

import ar.amgomez.idea.remotetail.logging.LogEntry;
import ar.amgomez.idea.remotetail.logging.LogLevel;
import org.jetbrains.annotations.NotNull;

/**
 * Class LogConsoleAlarm.
 * Keeps the state for the console's alarm.
 * Decides when the alarm is fired.
 * @author Alejandro Gomez
 */
public class LogConsoleAlarm {

    /**
     * The minimum log level required to fire the alarm.
     */
    private LogLevel logLevel;
    /**
     * The play sound flag.
     */
    private boolean playSound;
    /**
     * The popup flag.
     */
    private boolean popup;

    /**
     * Constructs a new <code>LogConsoleAlarm</code>
     */
    public LogConsoleAlarm() {
        logLevel = LogLevel.INFO;
    }

    /**
     * Sets the new <code>LogLevel</code>.
     * @param _logLevel the new value for the minimum log level required to fire the alarm.
     */
    public void setLogLevel(@NotNull final LogLevel _logLevel) {
        logLevel = _logLevel;
    }

    /**
     * Returns the value for the play sound flag.
     * @return true if a sound should be played when the alarm is fired.
     */
    public boolean isPlaySound() {
        return playSound;
    }

    /**
     * Sets the new value for the play sound flag.
     * @param _playSound whether a sound should be played or not when the alarm is fired.
     */
    public void setPlaySound(final boolean _playSound) {
        playSound = _playSound;
    }

    /**
     * Returns the value for the popup flag.
     * @return true if the tool window should be made visible when the alarm is fired.
     */
    public boolean isPopup() {
        return popup;
    }

    /**
     * Sets the new value for the popup flag.
     * @param _popup whether the tool window should be made visible or not when the alarm is fired.
     */
    public void setPopup(final boolean _popup) {
        popup = _popup;
    }

    /**
     * Checks if the alarm should be fired for the log entry.
     * @param _logEntry the log entry to check.
     * @return true if the play sound or popup flags are active and the log entry level is equal or greater than the minimum
     * required level to fire the alarm.
     */
    public boolean isApplicable(final LogEntry _logEntry) {
        return (playSound || popup) && _logEntry.isHeaderNonStrict(logLevel);
    }
}
