/*
 *    Copyright 2006 Alejandro Gomez
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 *
 */
package ar.amgomez.idea.remotetail;

import ar.amgomez.idea.remotetail.logging.LogEntry;
import ar.amgomez.idea.remotetail.logging.LogLevel;
import ar.amgomez.idea.util.GenericCollectionUtils;
import ar.amgomez.idea.util.GenericPredicate;

import java.util.ArrayList;
import java.util.List;

/**
 * Keeps the state for the console's filter.
 * Decides which log entries are printed.
 * @author Alejandro Gomez
 */
public class LogConsoleFilter {

    /**
     * The list of <code>LogLevel</code> being filtered.
     */
    private List<LogLevel> filteredLevels;

    /**
     * Constructs a new <code>LogConsoleFilter<code>.
     */
    public LogConsoleFilter() {
        filteredLevels = new ArrayList<LogLevel>();
    }

    /**
     * Adds a new level to the filtered levels.
     * @param _level the log level to add.
     */
    public void addLevel(final LogLevel _level) {
        filteredLevels.add(_level);
    }

    /**
     * Removes a level from the filtered levels.
     * @param _level the log level to remove.
     */
    public void removeLevel(final LogLevel _level) {
        filteredLevels.remove(_level);
    }

    /**
     * Selects and returns the unfiltered log entries.
     * @param _logEntries the log entries to filter.
     * @return a list with the unfiltered log entries.
     */
    public List<LogEntry> apply(final List<LogEntry> _logEntries) {
        return GenericCollectionUtils.select(_logEntries, new GenericPredicate<LogEntry>() {
            public boolean evaluate(final LogEntry _object) {
                return isUnfiltered(_object);
            }
        });
    }

    /**
     * Checks if the log entry passes or not the filter.
     * @param _logEntry the log entry to check.
     * @return true if the log entry is not filtered according to its level.
     */
    public boolean isUnfiltered(final LogEntry _logEntry) {
        return !filteredLevels.contains(_logEntry.getCategory().getLevel());
    }
}
