/*
 *    Copyright 2006 Alejandro Gomez
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 *
 */
package ar.amgomez.idea.remotetail;

import ar.amgomez.idea.remotetail.events.LogEntryListener;
import ar.amgomez.idea.remotetail.events.LogHeaderListener;
import ar.amgomez.idea.remotetail.logging.LogEntry;
import ar.amgomez.idea.remotetail.logging.LogLevel;

import java.util.List;

/**
 * Represents a log console.
 * @author Alejandro Gomez
 */
public interface LogConsoleModel {

    /**
     * Returns the log console title.
     * @return the log console title.
     */
    String getTitle();

    /**
     * Returns the host to which this console is connected.
     * @return the host to which this console is connected.
     */
    String getHost();

    /**
     * Returns the file path being tailed in this console.
     * @return the file path being tailed in this console.
     */
    String getFilePath();

    /**
     * Returns the list of unfiltered log entries.
     * @return the list of unfiltered log entries.
     */
    List<LogEntry> getUnfilteredLogEntries();

    /**
     * Adds a new <code>LogLevel</code> to this console's filters.
     * @param _logLevel the log level to add.
     */
    void addFilter(LogLevel _logLevel);

    /**
     * Removes a <code>LogLevel</code> from this console's filters.
     * @param _logLevel the log level to remove.
     */
    void removeFilter(LogLevel _logLevel);

    /**
     * Sets the new value for the alarm <code>LogLevel</code>.
     * @param _alarmLevel the new value for the alarm log leve.
     */
    void setAlarmLevel(LogLevel _alarmLevel);

    /**
     * Returns the value for the alarm's popup flag.
     * @return the value for the alarm's popup flag.
     */
    boolean isPopupOnAlarm();

    /**
     * Sets the new value for the popup flag for this console's alarm.
     * @param _popupOnAlarm the new value for the popup flag for this console's alarm.
     */
    void setPopupOnAlarm(boolean _popupOnAlarm);

    /**
     * Returns the value for the alarm's play sound flag.
     * @return the value for the alarm's play sound flag.
     */
    boolean isPlaySoundOnAlarm();

    /**
     * Sets the new value for the play sound flag for this console's alarm.
     * @param _playSoundOnAlarm the new value for the play sound flag for this console's alarm.
     */
    void setPlaySoundOnAlarm(boolean _playSoundOnAlarm);

    /**
     * Prints the console content.
     */
    void print();

    /**
     * Disposes this component.
     */
    void dispose();

    /**
     * Registers a new <code>LogEntryListener</code>.
     * @param _listener the <code>LogEntryListener</code> to register.
     */
    void addLogEntryListener(LogEntryListener _listener);

    /**
     * Unregisters a <code>LogEntryListener</code>.
     * @param _listener the <code>LogEntryListener</code> to unregister.
     */
    void removeLogEntryListener(LogEntryListener _listener);

    /**
     * Registers a new <code>LogHeaderListener</code>.
     * @param _listener the <code>LogHeaderListener</code> to register.
     */
    void addLogHeaderListener(LogHeaderListener _listener);

    /**
     * Unregisters a <code>LogHeaderListener</code>.
     * @param _listener the <code>LogHeaderListener</code> to unregister.
     */
    void removeLogHeaderListener(LogHeaderListener _listener);
}
