/*
 *    Copyright 2006 Alejandro Gomez
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 *
 */
package ar.amgomez.idea.remotetail;

import ar.amgomez.idea.remotetail.events.LogEntryListener;
import ar.amgomez.idea.remotetail.events.LogHeaderListener;
import ar.amgomez.idea.remotetail.logging.LogEntry;
import ar.amgomez.idea.remotetail.logging.LogEntryProvider;
import ar.amgomez.idea.remotetail.logging.LogLevel;
import ar.amgomez.idea.remotetail.logging.events.LogProviderListener;
import ar.amgomez.idea.util.LRAList;
import com.intellij.openapi.diagnostic.Logger;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

/**
 * Represents a log console.
 * @author Alejandro Gomez
 */
public class LogConsoleModelImpl implements LogConsoleModel {

    /**
     * To inform application errors using IDEA's logging mechanism.
     */
    private static final Logger LOGGER = Logger.getInstance(LogConsoleModelImpl.class.getName());

    /**
     * The log entry provider that feeds this console.
     */
    private LogEntryProvider provider;
    /**
     * The filter that decides which log entries are printed in this console.
     */
    private LogConsoleFilter filter;
    /**
     * The alarm for this console.
     */
    private LogConsoleAlarm alarm;
    /**
     * A list to keep the log entries generated by the provider.
     */
    private List<LogEntry> entries;
    /**
     * The <code>LogEntryListener</code> registered with this log console.
     */
    private final List<LogEntryListener> logEntryListeners;
    /**
     * The <code>LogHeaderListener</code> registered with this log console.
     */
    private final List<LogHeaderListener> logHeaderListeners;

    /**
     * Constructs a new <code>LogConsoleModel<code>
     * @param _logEntryProvider the log entry provider for the log console being created.
     * @param _bufferSize the buffer size for the entries to keep in memory.
     */
    public LogConsoleModelImpl(final LogEntryProvider _logEntryProvider, final int _bufferSize) {
        provider = _logEntryProvider;
        filter = new LogConsoleFilter();
        alarm = new LogConsoleAlarm();
        entries = new LRAList<LogEntry>(_bufferSize);
        logEntryListeners = new ArrayList<LogEntryListener>();
        logHeaderListeners = new ArrayList<LogHeaderListener>();
        provider.addLogProviderListener(new LogProviderListener() {
            public void logEntryAdded(final LogEntry _logEntry) {
                entries.add(_logEntry);
                if (filter.isUnfiltered(_logEntry)) {
                    fireUnfilteredLogEntryAdded(_logEntry);
                    if (alarm.isApplicable(_logEntry)) {
                        if (alarm.isPlaySound()) {
                            firePlaySoundRequest();
                        }
                        if (alarm.isPopup()) {
                            firePopupRequest();
                        }
                    }
                }
            }

            public void logProviderError(final String _message) {
                fireLogProviderError(_message);
            }

            public void logProviderException(final Exception _exception) {
                fireLogProviderException(_exception);
            }
        });
    }

    /**
     * Returns the log console title.
     * @return the log console title.
     */
    public String getTitle() {
        return provider.getShortDescription();
    }

    /**
     * Returns the host to which this console is connected.
     * @return the host to which this console is connected.
     */
    public String getHost() {
        return provider.getHost();
    }

    /**
     * Returns the file path being tailed in this console.
     * @return the file path being tailed in this console.
     */
    public String getFilePath() {
        return provider.getFilePath();
    }

    /**
     * Returns the list of unfiltered log entries.
     * @return the list of unfiltered log entries.
     */
    public List<LogEntry> getUnfilteredLogEntries() {
        return filter.apply(entries);
    }

    /**
     * Adds a <code>LogLevel</code> to this console's filters.
     * @param _logLevel the log level to add.
     */
    public void addFilter(final LogLevel _logLevel) {
        filter.addLevel(_logLevel);
        fireLogFilterChanged();
    }

    /**
     * Removes a <code>LogLevel</code> from this console's filters.
     * @param _logLevel the log level to remove.
     */
    public void removeFilter(final LogLevel _logLevel) {
        filter.removeLevel(_logLevel);
        fireLogFilterChanged();
    }

    /**
     * Sets the new value for the alarm <code>LogLevel</code>.
     * @param _alarmLevel the new value for the alarm log leve.
     */
    public void setAlarmLevel(final LogLevel _alarmLevel) {
        alarm.setLogLevel(_alarmLevel);
    }

    /**
     * Returns the value for the alarm's popup flag.
     * @return the value for the alarm's popup flag.
     */
    public boolean isPopupOnAlarm() {
        return alarm.isPopup();
    }

    /**
     * Sets the new value for the popup flag for this console's alarm.
     * @param _popupOnAlarm the new value for the popup flag for this console's alarm.
     */
    public void setPopupOnAlarm(final boolean _popupOnAlarm) {
        alarm.setPopup(_popupOnAlarm);
    }

    /**
     * Returns the value for the alarm's play sound flag.
     * @return the value for the alarm's play sound flag.
     */
    public boolean isPlaySoundOnAlarm() {
        return alarm.isPlaySound();
    }

    /**
     * Sets the new value for the play sound flag for this console's alarm.
     * @param _playSoundOnAlarm the new value for the play sound flag for this console's alarm.
     */
    public void setPlaySoundOnAlarm(final boolean _playSoundOnAlarm) {
        alarm.setPlaySound(_playSoundOnAlarm);
    }

    /**
     * Prints the console content.
     */
    public void print() {
        firePrintRequest();
    }

    /**
     * Disposes this component.
     */
    public void dispose() {
        logEntryListeners.clear();
        logHeaderListeners.clear();
        try {
            provider.disconnect();
        } catch (IOException ex) {
            LOGGER.info(ex);
        }
    }

    /**
     * Registers a new <code>LogEntryListener</code>.
     * @param _listener the <code>LogEntryListener</code> to register.
     */
    public synchronized void addLogEntryListener(final LogEntryListener _listener) {
        logEntryListeners.add(_listener);
    }

    /**
     * Unregisters a <code>LogEntryListener</code>.
     * @param _listener the <code>LogEntryListener</code> to unregister.
     */
    public synchronized void removeLogEntryListener(final LogEntryListener _listener) {
        logEntryListeners.remove(_listener);
    }

    /**
     * Registers a new <code>LogHeaderListener</code>.
     * @param _listener the <code>LogHeaderListener</code> to register.
     */
    public synchronized void addLogHeaderListener(final LogHeaderListener _listener) {
        logHeaderListeners.add(_listener);
    }

    /**
     * Unregisters a <code>LogHeaderListener</code>.
     * @param _listener the <code>LogHeaderListener</code> to unregister.
     */
    public synchronized void removeLogHeaderListener(final LogHeaderListener _listener) {
        logHeaderListeners.remove(_listener);
    }

    /**
     * Notifies the listeners about a new unfiltered log entry being added.
     * @param _unfilteredLogEntry the unfiltered log entry being added.
     */
    private synchronized void fireUnfilteredLogEntryAdded(final LogEntry _unfilteredLogEntry) {
        for (final LogEntryListener listener : logEntryListeners) {
            listener.unfilteredLogEntryAdded(_unfilteredLogEntry);
        }
    }

    /**
     * Notifies the listeners about a change in this console's filter.
     */
    private synchronized void fireLogFilterChanged() {
        for (final LogEntryListener listener : logEntryListeners) {
            listener.logFilterChanged();
        }
    }

    /**
     * Notifies the listeners about an error in the log entry provider.
     * @param _message the error message received from the log entry provider.
     */
    private void fireLogProviderError(final String _message) {
        for (final LogEntryListener listener : logEntryListeners) {
            listener.logProviderError(_message);
        }
    }

    /**
     * Notifies the listeners about an exception in the log entry provider.
     * @param _exception the exception received from the log entry provider.
     */
    private void fireLogProviderException(final Exception _exception) {
        for (final LogEntryListener listener : logEntryListeners) {
            listener.logProviderException(_exception);
        }
    }

    /**
     * Requests the listeners to play a sound.
     */
    private void firePlaySoundRequest() {
        for (final LogHeaderListener listener : logHeaderListeners) {
            listener.playSound(this);
        }
    }

    /**
     * Requests the listeners to popup this console.
     */
    private void firePopupRequest() {
        for (final LogHeaderListener listener : logHeaderListeners) {
            listener.popup(this);
        }
    }

    /**
     * Requests the listeners to print the content of this console.
     */
    private void firePrintRequest() {
        final List<LogEntry> unfilteredLogEntries = getUnfilteredLogEntries();
        for (final LogEntry logEntry : unfilteredLogEntries) {
            //TODO DocPrintJob
            System.out.println(logEntry);
        }
    }
}
