/*
 *    Copyright 2006 Alejandro Gomez
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 *
 */
package ar.amgomez.idea.remotetail;

import ar.amgomez.idea.remotetail.events.LogHeaderListener;
import ar.amgomez.idea.remotetail.events.MultipleConsolesListener;
import com.intellij.openapi.project.Project;
import org.jetbrains.annotations.Nullable;

import java.util.ArrayList;
import java.util.List;

/**
 * Represents a set of log consoles.
 * @author Alejandro Gomez
 */
public class MultipleConsolesModelImpl implements MultipleConsolesModel {

    /**
     * The project associated.
     */
    private Project project;
    /**
     * The index of the selected log console.
     */
    private int selectedLogConsoleIndex;
    /**
     * The list of the log consoles added.
     */
    private List<LogConsoleModel> logConsoleModels;
    /**
     * The listeners registered.
     */
    private final List<MultipleConsolesListener> listeners;

    /**
     * Constructs a new <code>MultipleConsolesModel<code>
     * @param _project the project associated with the model being created.
     */
    public MultipleConsolesModelImpl(final Project _project) {
        project = _project;
        selectedLogConsoleIndex = -1;
        logConsoleModels = new ArrayList<LogConsoleModel>();
        listeners = new ArrayList<MultipleConsolesListener>();
    }

    /**
     * Returns the project associated.
     * @return the project associated.
     */
    public Project getProject() {
        return project;
    }

    /**
     * Returns the selected log console.
     * @return the selected log console.
     */
    @Nullable
    public LogConsoleModel getSelectedLogConsole() {
        return selectedLogConsoleIndex == -1 ? null : logConsoleModels.get(selectedLogConsoleIndex);
    }

    /**
     * Selects a log console.
     * @param _index the index of the log console to select.
     */
    public void setSelectedLogConsole(final int _index) {
        if (selectedLogConsoleIndex != _index) {
            selectedLogConsoleIndex = _index;
            if (_index >= 0) {
                fireLogConsoleSelected(_index);
            }
        }
    }

    /**
     * Adds a new log console.
     * @param _logConsoleModel the log console to add.
     */
    public void addLogConsole(final LogConsoleModel _logConsoleModel) {
        logConsoleModels.add(_logConsoleModel);
        _logConsoleModel.addLogHeaderListener(new LogHeaderListener() {
            public void playSound(final LogConsoleModel _logConsoleModel) {
                selectLogConsole(_logConsoleModel);
                firePlaySound();
            }

            public void popup(final LogConsoleModel _logConsoleModel) {
                selectLogConsole(_logConsoleModel);
                firePopup();
            }

            private void selectLogConsole(final LogConsoleModel _logConsoleModel) {
                setSelectedLogConsole(logConsoleModels.indexOf(_logConsoleModel));
            }
        });
        fireLogConsoleAdded(_logConsoleModel);
    }

    /**
     * Removes the currently selected log console.
     */
    public void removeLogConsole() {
        removeLogConsole(selectedLogConsoleIndex);
    }

    /**
     * Removes all the log consoles.
     */
    public void removeAllConsoles() {
        for (int i = logConsoleModels.size() - 1; i >= 0; i--) {
            removeLogConsole(i);
        }
    }

    /**
     * Removes a log console indicated by its index.
     * @param _index the index of the log console being removed.
     */
    private void removeLogConsole(final int _index) {
        final LogConsoleModel logConsoleModel = logConsoleModels.remove(_index);
        logConsoleModel.dispose();
        fireLogConsoleRemoved(_index);
        if (logConsoleModels.isEmpty()) {
            setSelectedLogConsole(-1);
        } else {
            setSelectedLogConsole(_index < logConsoleModels.size() ? _index : _index - 1);
        }
    }

    /**
     * Disposes this component removing all listeners and the log consoles added.
     */
    public void dispose() {
        listeners.clear();
        removeAllConsoles();
    }

    /**
     * Registers a new <code>MultipleConsolesListener<code>.
     * @param _listener the <code>MultipleConsolesListener<code> to register.
     */
    public synchronized void addMultipleConsolesListener(final MultipleConsolesListener _listener) {
        listeners.add(_listener);
    }

    /**
     * Unregisters a <code>MultipleConsolesListener<code>.
     * @param _listener the <code>MultipleConsolesListener<code> to unregister.
     */
    public synchronized void removeMultipleConsolesListener(final MultipleConsolesListener _listener) {
        listeners.remove(_listener);
    }

    /**
     * Notifies the listeners about a log console being added.
     * @param _logConsoleAdded the log console being added.
     */
    private synchronized void fireLogConsoleAdded(final LogConsoleModel _logConsoleAdded) {
        for (final MultipleConsolesListener listener : listeners) {
            listener.consoleAdded(_logConsoleAdded);
        }
    }

    /**
     * Notifies the listeners about a log console being removed.
     * @param _index the index of the log console being removed.
     */
    private void fireLogConsoleRemoved(final int _index) {
        for (final MultipleConsolesListener listener : listeners) {
            listener.consoleRemoved(_index);
        }
    }

    /**
     * Notifies the listeners about a log console being selected.
     * @param _index the index of the log console being selected.
     */
    private void fireLogConsoleSelected(final int _index) {
        for (final MultipleConsolesListener listener : listeners) {
            listener.consoleSelected(_index);
        }
    }

    /**
     * Requests the listeners to popup.
     */
    private void firePopup() {
        for (final MultipleConsolesListener listener : listeners) {
            listener.popup();
        }
    }

    /**
     * Requests the listeners to play a sound.
     */
    private void firePlaySound() {
        for (final MultipleConsolesListener listener : listeners) {
            listener.playSound();
        }
    }
}
