/*
 *    Copyright 2006 Alejandro Gomez
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 *
 */
package ar.amgomez.idea.remotetail;

import ar.amgomez.idea.remotetail.events.MultipleConsolesAdapter;
import ar.amgomez.idea.remotetail.ui.ConsoleViewWrapper;
import ar.amgomez.idea.remotetail.ui.MultipleConsolesView;
import ar.amgomez.idea.remotetail.ui.MultipleConsolesViewImpl;
import com.intellij.execution.ui.ConsoleView;
import com.intellij.openapi.application.Application;
import com.intellij.openapi.application.ApplicationManager;
import com.intellij.openapi.application.ModalityState;
import com.intellij.openapi.project.Project;
import com.intellij.openapi.wm.ToolWindow;
import com.intellij.openapi.wm.ToolWindowAnchor;
import com.intellij.openapi.wm.ToolWindowManager;
import org.jetbrains.annotations.NotNull;
import org.picocontainer.MutablePicoContainer;
import org.picocontainer.Parameter;
import org.picocontainer.defaults.ComponentParameter;
import org.picocontainer.defaults.ConstructorInjectionComponentAdapterFactory;
import org.picocontainer.defaults.DefaultPicoContainer;

/**
 * the plugin implementation.
 * @author Alejandro Gomez
 */
public class RemoteTailPluginImpl implements RemoteTailPlugin {

    /**
     * The project associated to this component.
     */
    private Project project;
    /**
     * The tool window.
     */
    private ToolWindow toolWindow;
    /**
     * The container to resolve components dependencies.
     */
    private MutablePicoContainer container;

    /**
     * Constructs a new <code>RemoteTailPlugin</code>
     * @param _project the project associated with this component.
     */
    public RemoteTailPluginImpl(final Project _project) {
        project = _project;
    }

    /**
     * Initializes the component.
     */
    public void initComponent() {
        container = new DefaultPicoContainer(project.getPicoContainer());
        container.registerComponentImplementation(MultipleConsolesModel.class, MultipleConsolesModelImpl.class);
        container.registerComponentImplementation(MultipleConsolesView.class, MultipleConsolesViewImpl.class);
        //TODO use a different adapter instead of the wrapper
        container.registerComponent(new ConstructorInjectionComponentAdapterFactory().createComponentAdapter(ConsoleView.class, ConsoleViewWrapper.class, new Parameter[]{new ComponentParameter(Project.class)}));
        container.start();
    }

    /**
     * Disposes the resources.
     */
    public void disposeComponent() {
        container.stop();
        container.dispose();
    }

    /**
     * Method called when the project is opened.
     */
    public void projectOpened() {
        if (project != null && !project.isDefault()) {
            final MultipleConsolesModel multipleConsolesModel = getComponent(MultipleConsolesModel.class);
            final ToolWindowManager manager = ToolWindowManager.getInstance(project);
            final MultipleConsolesView multipleConsolesView = getComponent(MultipleConsolesView.class);
            toolWindow = manager.registerToolWindow(TOOL_WINDOW_ID, multipleConsolesView.getComponent(), ToolWindowAnchor.BOTTOM);
            toolWindow.setIcon(ICON);
            multipleConsolesModel.addMultipleConsolesListener(new MultipleConsolesAdapter() {
                @Override
                public void popup() {
                    final Application application = ApplicationManager.getApplication();
                    if (application.isDispatchThread()) {
                        toolWindow.show(null);
                    } else {
                        application.invokeAndWait(new Runnable() {
                            public void run() {
                                popup();
                            }
                        }, ModalityState.defaultModalityState());
                    }
                }

                @Override
                public void playSound() {
                    new Thread(new Runnable() {
                        public void run() {
                            RemoteTailHelper.getSettings().getAudioClip().play();
                        }
                    }).start();
                }
            });
        }
    }

    /**
     * Method called when the project is closed.
     */
    public void projectClosed() {
    }

    /**
     * Returns the component name.
     * @return the component name.
     */
    @NotNull
    public String getComponentName() {
        return COMPONENT_NAME;
    }

    /**
     * Returns a component implementation.
     * @param _interfaceClass the interface of the required component.
     * @return the component implementation.
     */
    public <T> T getComponent(final Class<T> _interfaceClass) {
        return (T)container.getComponentInstanceOfType(_interfaceClass);
    }
}
