/*
 *    Copyright 2006 Alejandro Gomez
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 *
 */
package ar.amgomez.idea.remotetail;

import ar.amgomez.idea.remotetail.settings.Settings;
import ar.amgomez.idea.remotetail.settings.SettingsImpl;
import ar.amgomez.idea.remotetail.ui.ConnectionForm;
import ar.amgomez.idea.remotetail.ui.SettingsForm;
import com.intellij.openapi.options.ConfigurationException;
import com.intellij.openapi.util.InvalidDataException;
import com.intellij.openapi.util.WriteExternalException;
import org.jdom.Element;
import org.jetbrains.annotations.Nls;
import org.jetbrains.annotations.NonNls;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import javax.swing.BorderFactory;
import javax.swing.Icon;
import javax.swing.JComponent;
import javax.swing.JPanel;
import java.awt.BorderLayout;

/**
 * The application component responsible for storing the configuration.
 * @author Alejandro Gomez
 */
public class RemoteTailSettingsImpl implements RemoteTailSettings {

    /**
     * The plugin settings.
     */
    private Settings settings;
    /**
     * The UI component for viewing and modifying the configuration.
     */
    private ConnectionForm connectionForm;

    /**
     * Constructs a new <code>RemoteTailSettings</code>
     */
    public RemoteTailSettingsImpl() {
        settings = new SettingsImpl();
    }

    /**
     * Initialize the component.
     */
    public void initComponent() {
    }

    /**
     * Disposes the component resources.
     */
    public void disposeComponent() {
    }

    /**
     * Returns the component name.
     * @return the component name.
     */
    @NotNull
    public String getComponentName() {
        return COMPONENT_NAME;
    }

    /**
     * Returns this component configuration.
     * @return the configuration values.
     */
    public Settings getSettings() {
        return settings;
    }

    /**
     * Reads the stored configuration.
     * @param _element the element containing the configuration data.
     * @throws InvalidDataException
     */
    public void readExternal(final Element _element) throws InvalidDataException {
        settings.readExternal(_element);
    }

    /**
     * Writes the configuration.
     * @param _element the element where to add the configuration data.
     * @throws WriteExternalException
     */
    public void writeExternal(final Element _element) throws WriteExternalException {
        settings.writeExternal(_element);
    }

    /**
     * Returns the user-visible name of the settings component.
     * @return the user-visible name of the settings component.
     */
    @Nls
    public String getDisplayName() {
        return DISPLAY_NAME;
    }

    /**
     * Returns the icon representing the settings component.
     * @return the icon representing the settings component.
     */
    public Icon getIcon() {
        return ICON;
    }

    /**
     * Returns the topic in the help file which is shown when help for the configurable is requested.
     * @return the help topic, or null if no help is available.
     */
    @Nullable
    @NonNls
    public String getHelpTopic() {
        return HELP_TOPIC;
    }

    /**
     * Returns the user interface component for editing the configuration.
     * @return the user interface component for editing the configuration.
     */
    public JComponent createComponent() {
        final JComponent component = new JPanel(new BorderLayout());
        final JPanel northPanel = new JPanel(new BorderLayout());
        northPanel.setBorder(BorderFactory.createTitledBorder(RemoteTailBundle.message("remotetail.settings.generalTitle")));
        northPanel.add(new SettingsForm(settings).getPanel(), BorderLayout.CENTER);
        component.add(northPanel, BorderLayout.NORTH);
        final JPanel centerPanel = new JPanel(new BorderLayout());
        centerPanel.setBorder(BorderFactory.createTitledBorder(RemoteTailBundle.message("remotetail.settings.connectionsTitle")));
        connectionForm = new ConnectionForm(settings);
        centerPanel.add(connectionForm.getPanel(), BorderLayout.CENTER);
        component.add(centerPanel, BorderLayout.CENTER);
        return component;
    }

    /**
     * Checks if the configuration in the user interface component were modified by the user and need to be saved.
     * @return true if the settings were modified, false otherwise.
     */
    public boolean isModified() {
        return settings.isModified();
    }

    /**
     * Stores the configuration.
     * @throws ConfigurationException
     */
    public void apply() throws ConfigurationException {
        settings.apply();
    }

    /**
     * Resets the configuration to the original values.
     */
    public void reset() {
        settings.reset();
    }

    /**
     * Disposes the Swing components used for displaying the configuration.
     */
    public void disposeUIResources() {
        connectionForm.dispose();
    }
}
