/*
 *    Copyright 2006 Alejandro Gomez
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 *
 */
package ar.amgomez.idea.remotetail.actions;

import ar.amgomez.idea.remotetail.LogConsoleModel;
import ar.amgomez.idea.remotetail.RemoteTailBundle;
import ar.amgomez.idea.remotetail.logging.LogLevel;
import com.intellij.openapi.actionSystem.AnAction;
import com.intellij.openapi.actionSystem.AnActionEvent;
import com.intellij.openapi.actionSystem.DefaultActionGroup;
import com.intellij.openapi.actionSystem.Presentation;
import com.intellij.openapi.actionSystem.ex.ComboBoxAction;
import org.jetbrains.annotations.NotNull;

import javax.swing.Icon;
import javax.swing.JComponent;

/**
 * A group of actions to choose the minimum log level that fires the alarms, if active.
 * @author Alejandro Gomez
 */
public class LevelComboBoxAction extends ComboBoxAction {

    /**
     * The log console model associated with this action.
     */
    private LogConsoleModel model;
    /**
     * The selected action.
     */
    private AlarmLevelAction selectedAlarmLevelAction;
    /**
     * The action group for grouping the three log levels actions.
     */
    private DefaultActionGroup alarmLevelActionGroup;
    /**
     * The presentation for this action group.
     */
    private Presentation presentation;

    /**
     * Creates a new action group for selecting the log level for the alarms.
     * @param _model the log console model associated with this action group.
     */
    public LevelComboBoxAction(final LogConsoleModel _model) {
        model = _model;
        final AlarmLevelAction infoAlarmLevelAction = new AlarmLevelAction(LogLevel.INFO, RemoteTailBundle.message("remotetail.alarmInfo.text"), RemoteTailBundle.message("remotetail.alarmInfo.description"), RemoteTailBundle.icon("remotetail.alarmInfo.icon"));
        selectedAlarmLevelAction = infoAlarmLevelAction;
        alarmLevelActionGroup = new DefaultActionGroup();
        alarmLevelActionGroup.add(infoAlarmLevelAction);
        alarmLevelActionGroup.add(new AlarmLevelAction(LogLevel.WARNING, RemoteTailBundle.message("remotetail.alarmWarning.text"), RemoteTailBundle.message("remotetail.alarmWarning.description"), RemoteTailBundle.icon("remotetail.alarmWarning.icon")));
        alarmLevelActionGroup.add(new AlarmLevelAction(LogLevel.ERROR, RemoteTailBundle.message("remotetail.alarmError.text"), RemoteTailBundle.message("remotetail.alarmError.description"), RemoteTailBundle.icon("remotetail.alarmError.icon")));
    }

    /**
     * Returns the custom component that representes this action UI.
     * @param _presentation the presentation.
     * @return the custom component that representes this action UI.
     */
    @Override
    public JComponent createCustomComponent(final Presentation _presentation) {
        presentation = _presentation;
        updatePresentation();
        return super.createCustomComponent(_presentation);
    }

    /**
     * Returns the group of log level actions.
     * @param _button the component.
     * @return the group of log level actions.
     */
    @Override
    @NotNull
    protected DefaultActionGroup createPopupActionGroup(final JComponent _button) {
        return alarmLevelActionGroup;
    }

    /**
     * Updates the presentation of this action with the presentation of the actually selected log level action.
     */
    private void updatePresentation() {
        final Presentation selectedPresentation = selectedAlarmLevelAction.getTemplatePresentation();
        presentation.setIcon(selectedPresentation.getIcon());
        presentation.setText(selectedPresentation.getText());
        presentation.setDescription(selectedPresentation.getDescription());
    }

    /**
     * Sets the selected log level action and updates the presentation.
     * @param _selectedAlarmLevelAction the selected log level action.
     */
    private void setSelectedAlarmLevel(final AlarmLevelAction _selectedAlarmLevelAction) {
        selectedAlarmLevelAction = _selectedAlarmLevelAction;
        updatePresentation();
        model.setAlarmLevel(selectedAlarmLevelAction.getAlarmLevel());
    }

    /**
     * Represents an action to select a specific log level for the alarms.
     * @author Alejandro Gomez
     */
    protected class AlarmLevelAction extends AnAction {

        /**
         * The alarm log level associated.
         */
        private LogLevel alarmLevel;

        /**
         * Creates a new action.
         * @param _alarmLevel the alarm log level associated.
         * @param _text the text to show.
         * @param _description the text that describes this action.
         * @param _icon the icon
         */
        protected AlarmLevelAction(final LogLevel _alarmLevel, final String _text, final String _description, final Icon _icon) {
            super(_text, _description, _icon);
            alarmLevel = _alarmLevel;
        }

        /**
         * The action handler.
         * Notifies the combo of actions about the new selection.
         * @param _event information about the invocation.
         */
        @Override
        public void actionPerformed(final AnActionEvent _event) {
            setSelectedAlarmLevel(this);
        }

        /**
         * Returns the log level associated with this action.
         * @return LogLevel
         */
        protected LogLevel getAlarmLevel() {
            return alarmLevel;
        }
    }
}
