/*
 *    Copyright 2006 Alejandro Gomez
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 *
 */
package ar.amgomez.idea.remotetail.logging;

import org.jetbrains.annotations.NonNls;

import java.util.regex.Pattern;

/**
 * The categories for the log entries.
 * @author Alejandro Gomez
 */
public enum LogCategory {

    /**
     * Unknown category.
     */
    UNKNOWN(null, null),
    /**
     * Info category.
     */
    INFO(LogLevel.INFO, ".*INFO.*"),
    /**
     * Debug category.
     */
    DEBUG(LogLevel.INFO, ".*DEBUG.*"),
    /**
     * Warning category.
     */
    WARNING(LogLevel.WARNING, ".*WARN.*"),
    /**
     * Error category.
     */
    ERROR(LogLevel.ERROR, ".*ERROR.*"),
    /**
     * Severe category.
     */
    SEVERE(LogLevel.ERROR, ".*SEVERE.*"),
    /**
     * Fatal category.
     */
    FATAL(LogLevel.ERROR, ".*FATAL.*");

    /**
     * The level associated with this category.
     */
    private LogLevel level;
    /**
     * The pattern to matches a <code>String</code> with this category.
     */
    private Pattern pattern;

    /**
     * Creates a new category.
     * @param _level the level associated with the new category.
     * @param _regex the patter associated with the new category.
     */
    LogCategory(final LogLevel _level, @NonNls final String _regex) {
        level = _level;
        if (_regex != null) {
            pattern = Pattern.compile(_regex);
        }
    }

    /**
     * Returns the level associated with this category.
     * @return the level associated with this category.
     */
    public LogLevel getLevel() {
        return level;
    }

    /**
     * Checks if the level of this category if greater or equal to the given level.
     * @param _level the level to compare.
     * @return true if the level of this category if greater or equal to the given level.
     */
    public boolean isLevelGreaterOrEqual(final LogLevel _level) {
        return this != UNKNOWN && level.compareTo(_level) >= 0;
    }

    /**
     * Checks if the given <code>String</code> matches this category's pattern.
     * @param _line the text to check.
     * @return true if the given <code>String</code> matches this category's pattern.
     */
    public boolean matches(final String _line) {
        return pattern != null && pattern.matcher(_line).matches();
    }
}
