/*
 *    Copyright 2006 Alejandro Gomez
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 *
 */
package ar.amgomez.idea.remotetail.logging;

import org.jetbrains.annotations.NonNls;

/**
 * A factory to create instances of <code>LogEntry</code>.
 * @author Alejandro Gomez
 */
public class LogEntryFactory {

    /**
     * The singleton instance of this factory.
     */
    private static final LogEntryFactory INSTANCE = new LogEntryFactory();

    /**
     * Private constructor to prevent other instances.
     */
    private LogEntryFactory() {
    }

    /**
     * Returns a new instance of <code>LogEntry</code>.
     * @param _text the text read by the provider.
     * @param _defaultCategory the default category for the new entry if none matches.
     * @return a new instance of <code>LogEntry</code>.
     */
    public LogEntry getLogEntry(@NonNls final String _text, final LogCategory _defaultCategory) {
        final LogCategory category = getLogCategory(_text, _defaultCategory);
        return new LogEntryImpl(_text, category, category.matches(_text));
    }

    /**
     * Returns the category matching the text or the default one if none matches.
     * @param _text the text read by the provider.
     * @param _defaultCategory the default category for the new entry if none matches.
     * @return the category matching the text or the default one if none matches.
     */
    public LogCategory getLogCategory(@NonNls final String _text, final LogCategory _defaultCategory) {
        for (final LogCategory logCategory : LogCategory.values()) {
            if (logCategory.matches(_text)) {
                return logCategory;
            }
        }
        return _defaultCategory;
    }

    /**
     * Returns the singleton instance of this factory.
     * @return the singleton instance of this factory.
     */
    public static LogEntryFactory getInstance() {
        return INSTANCE;
    }
}
