/*
 *    Copyright 2006 Alejandro Gomez
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 *
 */
package ar.amgomez.idea.remotetail.logging;

/**
 * Represents a log entry.
 * @author Alejandro Gomez
 */
public class LogEntryImpl implements LogEntry {

    /**
     * The text of this log entry.
     */
    private String text;
    /**
     * The category of this log entry.
     */
    private LogCategory category;
    /**
     * The header flag.
     */
    private boolean header;

    /**
     * Creates a new log entry.
     * @param _text the text of the new log entry.
     * @param _category the category of the new log entry.
     * @param _header the header flag of the new log entry.
     */
    protected LogEntryImpl(final String _text, final LogCategory _category, final boolean _header) {
        text = _text;
        category = _category;
        header = _header;
    }

    /**
     * Returns the text of this log entry.
     * @return the text of this log entry.
     */
    public String getText() {
        return text;
    }

    /**
     * Returns the category of this category.
     * @return the category of this category.
     */
    public LogCategory getCategory() {
        return category;
    }

    /**
     * Checks if this log entry is the header of a sequence of log entries.
     * The line matching the category's pattern is considered to be a header.
     * The subsequent lines (i.e. a stack trace for an exception) is not a header.
     * @return true if this log entry is the header of a sequence of log entries.
     */
    public boolean isHeader() {
        return header;
    }

    /**
     * Checks if this log entry is a header of the specified level.
     * @param _level the level.
     * @return true if this log entry is a header of the specified level.
     */
    public boolean isHeader(final LogLevel _level) {
        return getCategory().getLevel() == _level && isHeader();
    }

    /**
     * Checks if this log entry is a header of the specified level or a greter level.
     * @param _level the level.
     * @return true if this log entry is a header of the specified level or a greter level.
     */
    public boolean isHeaderNonStrict(final LogLevel _level) {
        return category.isLevelGreaterOrEqual(_level) && isHeader();
    }

    /**
     * Returns the <code>String</code> representation of this log entry.
     * @return the <code>String</code> representation of this log entry.
     */
    @Override
    public String toString() {
        return text;
    }

}
