/*
 *    Copyright 2006 Alejandro Gomez
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 *
 */
package ar.amgomez.idea.remotetail.logging;

import ar.amgomez.idea.remotetail.logging.events.LogProviderListener;
import ar.amgomez.idea.remotetail.net.Client;

import java.io.IOException;

/**
 * Interface representing an object that connects to a host and provides log entries.
 * @author Alejandro Gomez
 */
public interface LogEntryProvider {

    /**
     * Returns the network protocol client associated with this <code>LogEntryProvider</code>.
     * @return the network protocol client.
     */
    Client getClient();

    /**
     * Connects to the host.
     * @return true if the connection was successful.
     * @throws IOException if produced during the connection attempt.
     */
    boolean connect() throws IOException;

    /**
     * Disconnects from the host.
     * @throws IOException if produced during the disconnection attempt.
     */
    void disconnect() throws IOException;

    /**
     * Returns the name of the host to which this provider is connected.
     * @return the name of the host.
     */
    String getHost();

    /**
     * Returns the port number of the host to which this provider is connected.
     * @return the port number of the host.
     */
    int getPort();

    /**
     * Returns the name of the user connected to the host.
     * @return the name of the user connected to the host.
     */
    String getUsername();

    /**
     * Returns the path of the file which provides the log entries.
     * @return the path of the file.
     */
    String getFilePath();

    /**
     * Returns the name of the file which provides the log entries.
     * @return the name of the file.
     */
    String getFileName();

    /**
     * Returns a description for this log entry provider.
     * @return a description for this log entry provider.
     */
    String getDescription();

    /**
     * Returns a short description for this log entry provider.
     * @return a short description for this log entry provider.
     */
    String getShortDescription();

    /**
     * Registers new <code>LogProviderListener</code>.
     * @param _listener the <code>LogProviderListener</code> to register.
     */
    void addLogProviderListener(LogProviderListener _listener);

    /**
     * Unregisters a <code>LogProviderListener</code>.
     * @param _listener the <code>LogProviderListener</code> to unregister.
     */
    void removeLogProviderListener(LogProviderListener _listener);
}
