/*
 *    Copyright 2006 Alejandro Gomez
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 *
 */
package ar.amgomez.idea.remotetail.logging;

import ar.amgomez.idea.remotetail.logging.events.LogProviderListener;
import ar.amgomez.idea.remotetail.net.Client;
import org.jetbrains.annotations.NonNls;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.OutputStream;
import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * Implementation of a <code>LogEntryProvider</code>.
 * Provides listeners registration and event notification.
 * @author Alejandro Gomez
 */
public class LogEntryProviderImpl implements LogEntryProvider {

    /**
     * The pattern to generate the tail command.
     */
    @SuppressWarnings({"HardcodedLineSeparator"})
    @NonNls
    private static final String TAIL_PATTERN = "tail -f {0} &\n";
    /**
     * The message format to generate the tail command.
     */
    private static final MessageFormat TAIL_MESSAGE_FORMAT = new MessageFormat(TAIL_PATTERN);
    /**
     * The pattern to generate the kill command.
     */
    @SuppressWarnings({"HardcodedLineSeparator"})
    @NonNls
    private static final String KILL_PATTERN = "kill -9 {0}\n";
    /**
     * The message format to generate the kill command.
     */
    private static final MessageFormat KILL_MESSAGE_FORMAT = new MessageFormat(KILL_PATTERN);
    @SuppressWarnings({"HardcodedLineSeparator"})
    @NonNls
    private static final String EXIT = "exit\n";
    /**
     * The regular expression to parse the process id.
     */
    @SuppressWarnings({"HardcodedFileSeparator"})
    @NonNls
    private static final String PID_REG_EXP = "^.*\\[1\\] ((\\d)*)(\\s)*$";
    /**
     * The pattern to parse the process id.
     */
    private static final Pattern PID_PATTERN = Pattern.compile(PID_REG_EXP);

    /**
     * The network protocol client associated with this <code>LogEntryProvider</code>.
     */
    private Client client;
    /**
     * The listeners registered with this log entry provider.
     */
    private final List<LogProviderListener> listeners;
    /**
     * The category of the last log entry added.
     */
    private LogCategory lastCategory;
    /**
     * The tail process id.
     */
    private String pid;

    /**
     * Creates a log entry provider.
     * @param _client the connection info.
     */
    protected LogEntryProviderImpl(final Client _client) {
        client = _client;
        listeners = new ArrayList<LogProviderListener>();
        lastCategory = LogCategory.UNKNOWN;
    }

    /**
     * Returns the network protocol client associated with this <code>LogEntryProvider</code>.
     * @return the network protocol client.
     */
    public Client getClient() {
        return client;
    }

    /**
     * Connects to the host.
     *
     * @return true if the connection was successful.
     * @throws IOException if produced during the connection attempt.
     * @throws IllegalStateException if already connected.
     */
    public boolean connect() throws IOException {
        if (client.connect() && client.authenticate()) {
            new Thread(new InputStreamProcessor(client.getInputReader(), new LineProcessor() {
                public void process(final String _line) {
                    fireLogEntryAdded(_line);
                }
            }, true)).start();
            final BufferedReader errorReader = client.getErrorReader();
            if (errorReader != null) {
                new Thread(new InputStreamProcessor(errorReader, new LineProcessor() {
                    public void process(final String _line) {
                        fireLogProviderError(_line);
                    }
                })).start();
            }
            tail();
            return true;
        }
        return false;
    }

    /**
     * Sends the tail command to the host.
     * Starts the threads for reading the input and error streams.
     * The streams' lines read produce notifications on listeners.
     *
     * @throws IOException if produced during the command sending attempt.
     */
    private void tail() throws IOException {
        final OutputStream out = client.getOutputStream();
        out.write(getCommand().getBytes());
        out.flush();
    }

    /**
     * Disconnects from the host.
     * @throws IOException if produced during the disconnection attempt.
     */
    public void disconnect() throws IOException {
        final OutputStream out = client.getOutputStream();
        if (pid != null) {
            out.write(KILL_MESSAGE_FORMAT.format(new Object[]{pid}).getBytes());
        }
        out.write(EXIT.getBytes());
        out.flush();
        client.disconnect();
    }

    /**
     * Returns the name of the host to which this provider is connected.
     * @return the name of the host.
     */
    public String getHost() {
        return client.getConnectionInfo().getHost();
    }

    /**
     * Returns the port number of the host to which this provider is connected.
     * @return the port number of the host.
     */
    public int getPort() {
        return client.getConnectionInfo().getPort();
    }

    /**
     * Returns the name of the user connected to the host.
     * @return the name of the user connected to the host.
     */
    public String getUsername() {
        return client.getConnectionInfo().getUsername();
    }

    /**
     * Returns the path of the file which provides the log entries.
     * @return the path of the file.
     */
    public String getFilePath() {
        return client.getConnectionInfo().getFilePath();
    }

    /**
     * Returns the name of the file which provides the log entries.
     * @return the name of the file.
     */
    public String getFileName() {
        return client.getConnectionInfo().getFileName();
    }

    /**
     * Returns a description for this log entry provider.
     * @return a description for this log entry provider.
     */
    public String getDescription() {
        return client.getConnectionInfo().getDescription();
    }

    /**
     * Returns a short description for this log entry provider.
     * @return a short description for this log entry provider.
     */
    public String getShortDescription() {
        return client.getConnectionInfo().getShortDescription();
    }

    /**
     * Returns the command to execute on the host.
     * @return the command to execute on the host.
     */
    protected String getCommand() {
        return TAIL_MESSAGE_FORMAT.format(new Object[]{getFilePath()});
    }

    /**
     * Registers new <code>LogProviderListener</code>.
     * @param _listener the <code>LogProviderListener</code> to register.
     */
    public synchronized void addLogProviderListener(final LogProviderListener _listener) {
        listeners.add(_listener);
    }

    /**
     * Unregisters a <code>LogProviderListener</code>.
     * @param _listener the <code>LogProviderListener</code> to unregister.
     */
    public synchronized void removeLogProviderListener(final LogProviderListener _listener) {
        listeners.remove(_listener);
    }

    /**
     * Notifies the listeners about a new log entry added.
     * @param _message the text of the new line read.
     */
    protected synchronized void fireLogEntryAdded(final String _message) {
        final LogEntry logEntry = parseLine(_message);
        for (final LogProviderListener logProviderListener : listeners) {
            logProviderListener.logEntryAdded(logEntry);
        }
    }

    /**
     * Notifies the listeners about an error in the log entry provider.
     * @param _message the error message received from the log entry provider.
     */
    protected synchronized void fireLogProviderError(final String _message) {
        for (final LogProviderListener logProviderListener : listeners) {
            logProviderListener.logProviderError(_message);
        }
    }

    /**
     * Notifies the listeners about an exception in the log entry provider.
     * @param _exception the exception received from the log entry provider.
     */
    protected synchronized void fireLogProviderException(final Exception _exception) {
        for (final LogProviderListener logProviderListener : listeners) {
            logProviderListener.logProviderException(_exception);
        }
    }

    /**
     * Parses the line read transforming it to a new log entry.
     * @param _line the line read.
     * @return LogEntry the log entry generated from the line.
     */
    private LogEntry parseLine(final String _line) {
        final LogEntry logEntry = LogEntryFactory.getInstance().getLogEntry(_line, lastCategory);
        lastCategory = logEntry.getCategory();
        return logEntry;
    }

    /**
     * This class reads lines from a stream and process them.
     *
     * @author Alejandro Gomez
     */
    protected class InputStreamProcessor implements Runnable {

        /**
         * The reader to read from.
         */
        private BufferedReader reader;
        /**
         * The object that processes the lines read.
         */
        private LineProcessor lineProcessor;
        /**
         * Flag to indicate if this is the std out processor.
         */
        private boolean output;
        //private boolean firstLineSkipped;

        /**
         * Creates a new <code>InputStreamProcessor</code>.
         *
         * @param _reader the reader associated with the new <code>InputStreamProcessor</code>.
         * @param _lineProcessor the <code>LineProcessor</code> to process the lines.
         */
        protected InputStreamProcessor(final BufferedReader _reader, final LineProcessor _lineProcessor) {
            this(_reader, _lineProcessor, false);
        }

        /**
         * Creates a new <code>InputStreamProcessor</code>.
         *
         * @param _reader the reader associated with the new <code>InputStreamProcessor</code>.
         * @param _lineProcessor the <code>LineProcessor</code> to process the lines.
         * @param _isOutput indicates if this is the std out processor.
         */
        protected InputStreamProcessor(final BufferedReader _reader, final LineProcessor _lineProcessor, final boolean _isOutput) {
            reader = _reader;
            lineProcessor = _lineProcessor;
            output = _isOutput;
        }

        /**
         * Reads lines and process them.
         */
        public void run() {
            try {
                String line;
                while ((line = reader.readLine()) != null) {
                    if (output) {
                        if (pid == null) {
                            final Matcher matcher = PID_PATTERN.matcher(line);
                            if (matcher.matches()) {
                                pid = matcher.group(1);
                            }
                        } else /*if (firstLineSkipped)*/ {
                            lineProcessor.process(line);
/*
                        } else {
                            firstLineSkipped = true;
*/
                        }
                    } else {
                        lineProcessor.process(line);
                    }
                }
            } catch (IOException ex) {
                fireLogProviderException(ex);
            }
        }
    }

    /**
     * Represents a processor that executes actions on lines.
     *
     * @author Alejandro Gomez
     */
    protected static interface LineProcessor {

        /**
         * Process the line.
         *
         * @param _line the line.
         */
        void process(String _line);
    }
}
