/*
 *    Copyright 2006 Alejandro Gomez
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 *
 */
package ar.amgomez.idea.remotetail.net;

import ar.amgomez.idea.remotetail.settings.ConnectionInfo;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.OutputStream;

/**
 * Represents the network protocol client.
 * @author Alejandro Gomez
 */
public interface Client {

    /**
     * The socket time out.
     */
    int TIME_OUT = 10000;

    /**
     * Returns the <code>ConnectionInfo</code> associated with this client.
     * @return the <code>ConnectionInfo</code> associated with this client.
     */
    ConnectionInfo getConnectionInfo();

    /**
     * Checks if this client is connected to the remote host.
     * @return true if this client is connected to the remote host.
     */
    boolean isConnected();

    /**
     * Connects to the host.
     * @return true if the connection was successful.
     * @throws IOException if produced during the connection attempt.
     */
    boolean connect() throws IOException;

    /**
     * Authenticates the user at the host.
     * @return true if authentication was successful.
     * @throws IOException if produced during the connection attempt.
     */
    boolean authenticate() throws IOException;

    /**
     * Disconnects from the host.
     * @throws IOException if produced during the disconnection attempt.
     */
    void disconnect() throws IOException;

    /**
     * Returns the <code>OutputStream</code> associated to <code>stdin</code> in the remote host.
     *
     * @return the <code>OutputStream</code> for <code>stdin</code>.
     * @throws IOException if produced while getting the stream.
     */
    @NotNull
    OutputStream getOutputStream() throws IOException;

    /**
     * Returns the <code>BufferedReader</code> associated with <code>stdout</code> in the remote host.
     *
     * @return the <code>BufferedReader</code> for <code>stdout</code>.
     * @throws IOException if produced while getting the reader.
     */
    @NotNull
    BufferedReader getInputReader() throws IOException;

    /**
     * Returns the <code>BufferedReader</code> associated with <code>stderr</code> in the remote host.
     *
     * @return the <code>BufferedReader</code> for <code>stderr</code>.
     * @throws IOException if produced while getting the reader.
     */
    @Nullable
    BufferedReader getErrorReader() throws IOException;
}
