/*
 *    Copyright 2006 Alejandro Gomez
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 *
 */
package ar.amgomez.idea.remotetail.net;

import ar.amgomez.idea.remotetail.RemoteTailBundle;
import com.intellij.CommonBundle;
import com.intellij.openapi.application.ApplicationManager;
import com.intellij.openapi.application.ModalityState;
import com.intellij.openapi.ui.Messages;
import com.sshtools.j2ssh.transport.AbstractKnownHostsKeyVerification;
import com.sshtools.j2ssh.transport.InvalidHostFileException;
import com.sshtools.j2ssh.transport.TransportProtocolException;
import com.sshtools.j2ssh.transport.publickey.SshPublicKey;

import java.io.File;

/**
 * Class KnownHostsKeyVerification.
 * @author Alejandro Gomez
 */
public class KnownHostsKeyVerification extends AbstractKnownHostsKeyVerification {

    /**
     * Constructs a host key verification instance reading the known_hosts file in the user home dir.
     * @throws InvalidHostFileException if the known_hosts file is invalid
     */
    public KnownHostsKeyVerification() throws InvalidHostFileException {
        super(new File(new File(System.getProperty("user.home", ".ssh")), "known_hosts").getAbsolutePath());
    }

    public boolean verifyHost(final String _host, final SshPublicKey _key) throws TransportProtocolException {
        try {
            return super.verifyHost(_host, _key);
        } catch (UserDeniedException ex) {
            return false;
        }
    }

    /**
     * Called by the <code>verifyHost</code> method when the host key supplied
     * by the _host does not match the current key recording in the known hosts
     * file.
     * @param _host           the name of the host
     * @param _allowedHostKey the current key recorded in the known_hosts file.
     * @param _actualHostKey  the actual key supplied by the user
     * @throws TransportProtocolException if an error occurs
     */
    @Override
    public void onHostKeyMismatch(final String _host, final SshPublicKey _allowedHostKey, final SshPublicKey _actualHostKey) throws TransportProtocolException {
        getResponse(
                RemoteTailBundle.message("remotetail.connection.fingerprint.mismatch.message", _host, _allowedHostKey.getFingerprint(), _allowedHostKey.getFingerprint()),
                RemoteTailBundle.message("remotetail.connection.fingerprint.mismatch.title"),
                _host, _actualHostKey
        );
    }

    /**
     * Called by the <code>verifyHost</code> method when the host key supplied
     * is not recorded in the known_hosts file.
     * @param _host the name of the host
     * @param _key  the public key supplied by the host
     * @throws TransportProtocolException if an error occurs
     */
    @Override
    public void onUnknownHost(final String _host, final SshPublicKey _key) throws TransportProtocolException {
        getResponse(
                RemoteTailBundle.message("remotetail.connection.fingerprint.unknown.message", _host, _key.getFingerprint()),
                RemoteTailBundle.message("remotetail.connection.fingerprint.unknown.title"),
                _host, _key
        );
    }

    private void getResponse(final String _message, final String _title, final String _host, final SshPublicKey _key) throws TransportProtocolException {
        final int[] result = {0};
        ApplicationManager.getApplication().invokeAndWait(new Runnable() {
            public void run() {
                result[0] = Messages.showDialog(_message, _title, new String[]{CommonBundle.getYesButtonText(), RemoteTailBundle.message("remotetail.always"), CommonBundle.getNoButtonText()}, 0, Messages.getQuestionIcon());
            }
        }, ModalityState.NON_MODAL);
        switch (result[0]) {
            case 0:
                allowHost(_host, _key, false);
                break;
            case 1:
                allowHost(_host, _key, true);
                break;
            default:
                throw new UserDeniedException();
        }
    }

    private static class UserDeniedException extends TransportProtocolException {

        public UserDeniedException() {
            super("User cancelled.");
        }
    }
}
