/*
 *    Copyright 2006 Alejandro Gomez
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 *
 */
package ar.amgomez.idea.remotetail.net;

import ar.amgomez.idea.remotetail.settings.ConnectionInfo;
import com.intellij.openapi.diagnostic.Logger;
import com.sshtools.j2ssh.SftpClient;
import com.sshtools.j2ssh.SshClient;
import com.sshtools.j2ssh.authentication.AuthenticationProtocolState;
import com.sshtools.j2ssh.authentication.PasswordAuthenticationClient;
import com.sshtools.j2ssh.authentication.PublicKeyAuthenticationClient;
import com.sshtools.j2ssh.authentication.SshAuthenticationClient;
import com.sshtools.j2ssh.configuration.SshConnectionProperties;
import com.sshtools.j2ssh.connection.Channel;
import com.sshtools.j2ssh.session.SessionChannelClient;
import com.sshtools.j2ssh.transport.HostKeyVerification;
import com.sshtools.j2ssh.transport.IgnoreHostKeyVerification;
import com.sshtools.j2ssh.transport.publickey.SshPrivateKeyFile;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.io.*;
import java.util.List;

/**
 * Represents the SSH network protocol client.
 * @author Alejandro Gomez
 */
public class SSHClient implements Client {

    /**
     * To inform application errors using IDEA's logging mechanism.
     */
    private static final Logger LOGGER = Logger.getInstance(SSHClient.class.getName());

    /**
     * The <code>ConnectionInfo</code> associated with this <code>Client</code>.
     */
    private ConnectionInfo connectionInfo;
    /**
     * The SSH client.
     */
    private SshClient client;
    /**
     * The session channel client.
     */
    private SessionChannelClient sessionChannelClient;
    /**
     * The SFTP client.
     */
    private SftpClient sftpClient;

    /**
     * Creates a new <code>SSHClient</code>.
     * @param _connectionInfo the <code>ConnectionInfo</code> associated with this client.
     */
    public SSHClient(final ConnectionInfo _connectionInfo) {
        connectionInfo = new ConnectionInfo(_connectionInfo);
    }

    /**
     * Returns the <code>ConnectionInfo</code> associated with this client.
     * @return the <code>ConnectionInfo</code> associated with this client.
     */
    public ConnectionInfo getConnectionInfo() {
        return connectionInfo;
    }

    /**
     * Checks if this client is connected to the remote host.
     * @return true if this client is connected to the remote host.
     */
    public boolean isConnected() {
        return client.isConnected();
    }

    /**
     * Connects to the host.
     * @return true if the connection was successful.
     * @throws IOException if produced during the connection attempt.
     */
    public boolean connect() throws IOException {
        if (client != null && client.isConnected()) {
            throw new IllegalStateException("Already connected");
        }
        client = new SshClient();
        client.setSocketTimeout(TIME_OUT);
        client.connect(getSshConnectionProperties(), getHostKeyVerification());
        return true;
    }

    /**
     * Authenticates the user at the host.
     * @return true if authentication was successful.
     * @throws IOException if produced during the connection attempt.
     */
    public boolean authenticate() throws IOException {
        if (client == null || !client.isConnected()) {
            throw new IllegalStateException("Not connected");
        }
        return client.authenticate(getAuthenticationClient()) == AuthenticationProtocolState.COMPLETE;
    }

    /**
     * Disconnects from the host.
     * @throws IOException if produced during the disconnection attempt.
     */
    public void disconnect() throws IOException {
        try {
            final List activeChannels = client.getActiveChannels();
            for (final Object activeChannel : activeChannels) {
                ((Channel)activeChannel).close();
            }
        } finally {
            try {
                if (sftpClient != null) {
                    sftpClient.quit();
                }
            } finally {
                if (client != null) {
                    client.disconnect();
                }
            }
        }
    }

    /**
     * Returns the <code>OutputStream</code> associated to <code>stdin</code> in the remote host.
     *
     * @return the <code>OutputStream</code> for <code>stdin</code>.
     * @throws IOException if produced while getting the stream.
     */
    @NotNull
    public OutputStream getOutputStream() throws IOException {
        return getSessionChannelClient().getOutputStream();
    }

    /**
     * Returns the <code>BufferedReader</code> associated with <code>stdout</code> in the remote host.
     *
     * @return the <code>BufferedReader</code> for <code>stdout</code>.
     * @throws IOException if produced while getting the reader.
     */
    @NotNull
    public BufferedReader getInputReader() throws IOException {
        return new BufferedReader(new InputStreamReader(getSessionChannelClient().getInputStream()));
    }

    /**
     * Returns the <code>BufferedReader</code> associated with <code>stderr</code> in the remote host.
     *
     * @return the <code>BufferedReader</code> for <code>stderr</code>.
     * @throws IOException if produced while getting the reader.
     */
    @Nullable
    public BufferedReader getErrorReader() throws IOException {
        return new BufferedReader(new InputStreamReader(getSessionChannelClient().getStderrInputStream()));
    }

    /**
     * Returns the <code>SessionChannelClient</code>.
     * If the client is not open, opens a new one.
     * @return the <code>SessionChannelClient</code>.
     * @throws IOException if produced while opening the <code>SftpSubsystemClient</code>.
     * @throws IllegalArgumentException if this client is not connected.
     */
    private SessionChannelClient getSessionChannelClient() throws IOException {
        if (client == null || !client.isConnected()) {
            throw new IllegalStateException("Not connected");
        }
        if (sessionChannelClient == null) {
            sessionChannelClient = client.openSessionChannel();
            //to read the pid, makes stderr not to work :(
            sessionChannelClient.requestPseudoTerminal("vt100", 80, 24, 0, 0, "");
            sessionChannelClient.startShell();
        }
        return sessionChannelClient;
    }

    /**
     * Returns the <code>SftpClient</code>.
     * If the client is not open, opens a new one.
     * @return the <code>SftpClient</code>.
     * @throws IOException if produced while opening the <code>SftpClient</code>.
     * @throws IllegalArgumentException if this client is not connected.
     */
    public SftpClient getSftpClient() throws IOException {
        if (client == null || !client.isConnected()) {
            throw new IllegalStateException("Not connected");
        }
        if (sftpClient == null) {
            sftpClient = client.openSftpClient();
        }
        return sftpClient;
    }

    /**
     * Creates the connection properties.
     * @return the connection properties.
     */
    private SshConnectionProperties getSshConnectionProperties() {
        final SshConnectionProperties properties = new SshConnectionProperties();
        properties.setHost(connectionInfo.getHost());
        properties.setPort(connectionInfo.getPort());
        return properties;
    }

    /**
     * Creates the authentication client.
     * @return the authentication client.
     * @throws IOException if an I/O error is produced.
     */
    private SshAuthenticationClient getAuthenticationClient() throws IOException {
        if (connectionInfo.isUsePassword()) {
            final PasswordAuthenticationClient authenticationClient = new PasswordAuthenticationClient();
            authenticationClient.setUsername(connectionInfo.getUsername());
            authenticationClient.setPassword(connectionInfo.getPassword());
            return authenticationClient;
        } else {
            final PublicKeyAuthenticationClient authenticationClient = new PublicKeyAuthenticationClient();
            authenticationClient.setUsername(connectionInfo.getUsername());
            final SshPrivateKeyFile privateKeyFile = SshPrivateKeyFile.parse(new File(connectionInfo.getPrivateKeyFilePath()));
            authenticationClient.setKey(privateKeyFile.toPrivateKey(connectionInfo.getPassphrase()));
            return authenticationClient;
        }
    }

    /**
     * Creates an instance of a host key verificator.
     * The returned instance always verifies the host.
     * @return the host key verificator.
     */
    private static HostKeyVerification getHostKeyVerification() {
        return new IgnoreHostKeyVerification();
/*
        try {
            return new KnownHostsKeyVerification();
        } catch (InvalidHostFileException ex) {
            LOGGER.info(ex);
            return new IgnoreHostKeyVerification();
        }
*/
    }
}
