/*
 *    Copyright 2006 Alejandro Gomez
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 *
 */
package ar.amgomez.idea.remotetail.net;

import com.sshtools.j2ssh.sftp.SftpFile;

/**
 * Represents a <code>RemoteFile</code> read via the SSH protocol.
 * @author Alejandro Gomez
 */
public class SSHRemoteFile implements RemoteFile {

    /**
     * The encapsulated <code>SftpFile</code>.
     */
    private SftpFile file;
    /**
     * The flag that indicates if this file can be read.
     */
    private boolean canRead;

    /**
     * Constructs a new <code>SSHRemoteFile</code>.
     * @param _file the <code>SftpFile</code> to encapsulate.
     */
    public SSHRemoteFile(final SftpFile _file) {
        file = _file;
        canRead = _file.canRead();
    }

    /**
     * Returns the file name.
     * @return the file name.
     */
    public String getFilename() {
        return file.getFilename();
    }

    /**
     * Returns the absolute path of the file.
     * @return String
     */
    public String getAbsolutePath() {
        return file.getAbsolutePath();
    }

    /**
     * Checks if the file is a directory.
     * @return true if the file is a directory.
     */
    public boolean isDirectory() {
        return file.isDirectory();
    }

    /**
     * Checks if it is a file.
     * @return true if it is a file.
     */
    public boolean isFile() {
        return file.isFile();
    }

    /**
     * Checks if the file can be read.
     * @return true if the file can be read.
     */
    public boolean canRead() {
        return canRead;
    }

    /**
     * Sets the new value for the file readability.
     * @param _canRead the new value for the file readability.
     */
    public void setCanRead(final boolean _canRead) {
        canRead = _canRead;
    }

    /**
     * Checks if the file is a pointer to the actual or the parent directory (. or ..).
     * @return true if the file is a pointer.
     * @see RemoteFile#THIS_POINTER
     * @see RemoteFile#PARENT_POINTER
     */
    public boolean isPointer() {
        return THIS_POINTER.equals(getFilename()) || PARENT_POINTER.equals(getFilename());
    }

    /**
     * Compares this file to another.
     * @param   _object the <code>Object</code> to be compared.
     * @return  a negative integer, zero, or a positive integer as this object
     *		is less than, equal to, or greater than the specified object.
     * @throws ClassCastException if the specified object is not a <code>RemoteFile</code>.
     */
    public int compareTo(final Object _object) {
        final RemoteFile other = (RemoteFile)_object;
        if (isDirectory() && !other.isDirectory()) {
            return -1;
        } else if (!isDirectory() && other.isDirectory()) {
            return 1;
        } else {
            return getFilename().compareTo(other.getFilename());
        }
    }
}
