/*
 *    Copyright 2006 Alejandro Gomez
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 *
 */
package ar.amgomez.idea.remotetail.net;

import ar.amgomez.idea.remotetail.settings.ConnectionInfo;
import org.apache.commons.net.telnet.EchoOptionHandler;
import org.apache.commons.net.telnet.InvalidTelnetOptionException;
import org.apache.commons.net.telnet.SuppressGAOptionHandler;
import org.apache.commons.net.telnet.TerminalTypeOptionHandler;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStream;

/**
 * Class TelnetClient.
 * @author Alejandro Gomez
 */
public class TelnetClient implements Client {

    /**
     * The <code>ConnectionInfo</code> associated with this <code>Client</code>.
     */
    private ConnectionInfo connectionInfo;
    /**
     * The <code>TelnetClient</code>.
     */
    private org.apache.commons.net.telnet.TelnetClient client;
    /**
     * The <code>OutputStream</code>.
     */
    private OutputStream out;
    /**
     * The <code>BufferedReader</code>.
     */
    private BufferedReader reader;

    /**
     * Creates a new <code>TelnetClient</code>.
     * @param _connectionInfo the <code>ConnectionInfo</code> associated with this client.
     */
    public TelnetClient(final ConnectionInfo _connectionInfo) {
        connectionInfo = _connectionInfo;
    }

    /**
     * Returns the <code>ConnectionInfo</code> associated with this client.
     * @return the <code>ConnectionInfo</code> associated with this client.
     */
    public ConnectionInfo getConnectionInfo() {
        return connectionInfo;
    }

    /**
     * Checks if this client is connected to the remote host.
     * @return true if this client is connected to the remote host.
     */
    public boolean isConnected() {
        return client.isConnected();
    }

    /**
     * Connects to the host.
     * @return true if the connection was successful.
     * @throws IOException if produced during the connection attempt.
     */
    public boolean connect() throws IOException {
        if (client != null && client.isConnected()) {
            throw new IllegalStateException("Already connected");
        }
        client = new org.apache.commons.net.telnet.TelnetClient();
        client.setDefaultTimeout(TIME_OUT);
        try {
            client.addOptionHandler(new TerminalTypeOptionHandler("VT100", false, false, true, false));
            client.addOptionHandler(new EchoOptionHandler(true, false, true, false));
            client.addOptionHandler(new SuppressGAOptionHandler(true, true, true, true));
        } catch (InvalidTelnetOptionException ex) {
            throw new IOException(ex.getMessage());
        }
        client.connect(connectionInfo.getHost(), connectionInfo.getPort());
        return true;
    }

    /**
     * Authenticates the user at the host.
     * @return true if authentication was successful.
     * @throws IOException if produced during the connection attempt.
     */
    public boolean authenticate() throws IOException {
        reader = new BufferedReader(new InputStreamReader(client.getInputStream()));
        out = client.getOutputStream();
        readInputLine();
        out.write(connectionInfo.getUsername().getBytes());
        out.write('\n');
        out.flush();
        readInputLine();
        out.write(connectionInfo.getPassword().getBytes());
        out.write('\n');
        out.flush();
        return true;
    }

    private void readInputLine() throws IOException {
        final StringBuilder builder = new StringBuilder();
        final byte[] buffer = new byte[1024];
        do {
            final int length = client.getInputStream().read(buffer);
            if (length == -1) {
                throw new IOException("can not read.");
            }
            builder.append(new String(buffer, 0, length));
        } while (!builder.toString().endsWith(": "));
    }

    /**
     * Disconnects from the host.
     * @throws IOException if produced during the disconnection attempt.
     */
    public void disconnect() throws IOException {
        try {
            if (out != null) {
                out.close();
            }
        } finally {
            try {
                if (reader != null) {
                    reader.close();
                }
            } finally {
                if (client != null) {
                    client.disconnect();
                }
            }
        }
    }

    /**
     * Returns the <code>OutputStream</code> associated to <code>stdin</code> in the remote host.
     *
     * @return the <code>OutputStream</code> for <code>stdin</code>.
     * @throws IOException if produced while getting the stream.
     */
    @NotNull
    public OutputStream getOutputStream() throws IOException {
        if (client == null || !client.isConnected()) {
            throw new IllegalStateException("Not connected");
        }
        if (out == null) {
            out = client.getOutputStream();
        }
        return out;
    }

    /**
     * Returns the <code>BufferedReader</code> associated with <code>stdout</code> in the remote host.
     *
     * @return the <code>BufferedReader</code> for <code>stdout</code>.
     * @throws IOException if produced while getting the reader.
     */
    @NotNull
    public BufferedReader getInputReader() throws IOException {
        if (client == null || !client.isConnected()) {
            throw new IllegalStateException("Not connected");
        }
        if (reader == null) {
            reader = new BufferedReader(new InputStreamReader(client.getInputStream()));
        }
        return reader;
    }

    /**
     * Returns the <code>BufferedReader</code> associated with <code>stderr</code> in the remote host.
     *
     * @return the <code>BufferedReader</code> for <code>stderr</code>.
     * @throws IOException if produced while getting the reader.
     */
    @Nullable
    public BufferedReader getErrorReader() throws IOException {
        return null;
    }
}
