/*
 *    Copyright 2006 Alejandro Gomez
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 *
 */
package ar.amgomez.idea.remotetail.settings;

import ar.amgomez.idea.remotetail.net.Protocol;
import ar.amgomez.idea.util.StringUtils;
import com.intellij.openapi.util.text.StringUtil;
import org.jetbrains.annotations.NonNls;
import org.jetbrains.annotations.Nullable;

import java.io.File;

/**
 * Contains parameters for the connection.
 * @author Alejandro Gomez
 */
public class ConnectionInfo {

    /**
     * The host name.
     */
    @NonNls
    private String host;
    /**
     * The port number.
     */
    private int port;
    /**
     * The network protocol.
     */
    private Protocol protocol;
    /**
     * The file path.
     */
    @NonNls
    private String filePath;
    /**
     * The username.
     */
    @NonNls
    private String username;
    /**
     * The password authentication method flag.
     */
    private boolean usePassword;
    /**
     * The password.
     */
    @NonNls
    private String password;
    /**
     * The private key file path.
     */
    @NonNls
    private String privateKeyFilePath;
    /**
     * The private key passphrase.
     */
    @NonNls
    private String passphrase;

    /**
     * Creates a new <code>ConnectionInfo</code>.
     */
    public ConnectionInfo() {
        protocol = Protocol.SSH;
        port = protocol.getDefaultPort();
        usePassword = true;
    }

    /**
     * Creates a new <code>ConnectionInfo</code> copying the parameters from another <code>ConnectionInfo</code>.
     * @param _other the other <code>ConnectionInfo</code>.
     */
    public ConnectionInfo(final ConnectionInfo _other) {
        host = _other.getHost();
        port = _other.getPort();
        protocol = _other.getProtocol();
        filePath = _other.getFilePath();
        username = _other.getUsername();
        usePassword = _other.isUsePassword();
        password = _other.getPassword();
        privateKeyFilePath = _other.getPrivateKeyFilePath();
        passphrase = _other.getPassphrase();
    }

    /**
     * Returns a description for this log entry provider.
     * @return a description for this log entry provider.
     */
    @Nullable
    public String getDescription() {
        return isEmpty() ? null : username + '@' + host + ':' + port + ':' + filePath;
    }

    /**
     * Returns a short description for this log entry provider.
     * @return a short description for this log entry provider.
     */
    @Nullable
    public String getShortDescription() {
        return isEmpty() ? null : getFileName() + '@' + host;
    }

    /**
     * Returns the host.
     * @return the host value.
     */
    public String getHost() {
        return host;
    }

    /**
     * Sets the new value for the host.
     * @param _host the new value for the host.
     */
    public void setHost(final String _host) {
        host = _host;
    }

    /**
     * Returns the port number.
     * @return the port number.
     */
    public int getPort() {
        return port;
    }

    /**
     * Sets the new value for the port.
     * @param _port the new value for the port.
     */
    public void setPort(final int _port) {
        port = _port;
    }

    /**
     * Returns the network protocol.
     * @return the network protocol.
     */
    public Protocol getProtocol() {
        return protocol;
    }

    /**
     * Sets the new value for the network protocol.
     * @param _protocol the new value for the network protocol.
     */
    public void setProtocol(final Protocol _protocol) {
        protocol = _protocol;
    }

    /**
     * Returns the username.
     * @return the username.
     */
    public String getUsername() {
        return username;
    }

    /**
     * Sets the new value for the username.
     * @param _username the new value for the username.
     */
    public void setUsername(final String _username) {
        username = _username;
    }

    /**
     * Returns the file path.
     * @return the file path.
     */
    public String getFilePath() {
        return filePath;
    }

    /**
     * Sets the new value for the file path.
     * @param _filePath the new value for the file path.
     */
    public void setFilePath(final String _filePath) {
        filePath = _filePath;
    }

    /**
     * Returns the value for the password authentication method flag.
     * @return true if the authentication should be done using the password.
     */
    public boolean isUsePassword() {
        return usePassword;
    }

    /**
     * Sets the new value for the password authentication method flag.
     * @param _usePassword the new value for the password authentication method flag.
     */
    public void setUsePassword(final boolean _usePassword) {
        usePassword = _usePassword;
    }

    /**
     * Returns the value for the password.
     * @return the value for the password.
     */
    public String getPassword() {
        return password;
    }

    /**
     * Sets the new value for the password.
     * @param _password the new value for the password.
     */
    public void setPassword(final String _password) {
        password = _password;
    }

    /**
     * Returns the private key file path.
     * @return the private key file path.
     */
    public String getPrivateKeyFilePath() {
        return privateKeyFilePath;
    }

    /**
     * Sets the new file path for the private key.
     * @param _privateKeyFilePath the new file path for the private key.
     */
    public void setPrivateKeyFilePath(final String _privateKeyFilePath) {
        privateKeyFilePath = _privateKeyFilePath;
    }

    /**
     * Returns the value for the passphrase.
     * @return the value for the passphrase.
     */
    public String getPassphrase() {
        return passphrase;
    }

    /**
     * Sets the new value for the passphrase.
     * @param _passphrase the new value for the passphrase.
     */
    public void setPassphrase(final String _passphrase) {
        passphrase = _passphrase;
    }

    /**
     * Returns the file name.
     * @return the file name.
     */
    public String getFileName() {
        return new File(filePath).getName();
    }

    /**
     * Checks if the parameters are valid.
     * @return true if the parameters are valid.
     */
    public boolean isValid() {
        return isConnectable() && StringUtil.isNotEmpty(filePath);
    }

    /**
     * Checks if all needed parameters to connect are completed.
     * @return true if all needed parameters to connect are completed.
     */
    public boolean isConnectable() {
        return StringUtil.isNotEmpty(host) && port > 0 && StringUtil.isNotEmpty(username) && protocol != null &&
                (usePassword && StringUtil.isNotEmpty(password) || !usePassword && StringUtil.isNotEmpty(privateKeyFilePath));
    }

    /**
     * Checks if all the remote file system can be navigated.
     * @return true if the remote file system can be navigated.
     */
    public boolean canNavigate() {
        return isConnectable() && protocol == Protocol.SSH;
    }

    /**
     * Checks if all the parameters are empty.
     * @return true if all the parameters are empty.
     */
    public boolean isEmpty() {
        return StringUtil.isEmpty(host) &&
                StringUtil.isEmpty(username) &&
                StringUtil.isEmpty(filePath) &&
                StringUtil.isEmpty(password) &&
                StringUtil.isEmpty(privateKeyFilePath) &&
                StringUtil.isEmpty(passphrase);
    }

    /**
     * Checks if the other object is equal to this.
     * @param _object the other object.
     * @return true if the other object is equal to this.
     */
    @Override
    public boolean equals(final Object _object) {
        if (_object == this) {
            return true;
        } else if (_object instanceof ConnectionInfo) {
            final ConnectionInfo other = (ConnectionInfo)_object;
            return StringUtils.equals(host, other.getHost()) &&
                    port == other.getPort() &&
                    protocol == other.getProtocol() &&
                    StringUtils.equals(username, other.getUsername()) &&
                    StringUtils.equals(filePath, other.getFilePath()) &&
                    usePassword == other.isUsePassword() &&
                    StringUtils.equals(password, other.getPassword()) &&
                    StringUtils.equals(privateKeyFilePath, other.getPrivateKeyFilePath()) &&
                    StringUtils.equals(passphrase, other.getPassphrase());
        } else {
            return false;
        }
    }
}
