/*
 *    Copyright 2006 Alejandro Gomez
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 *
 */
package ar.amgomez.idea.remotetail.settings;

import com.intellij.openapi.util.JDOMExternalizable;
import org.jetbrains.annotations.NonNls;
import org.jetbrains.annotations.NotNull;

import java.applet.AudioClip;
import java.util.List;

/**
 * Interface for the definition and modification of the plugin configuration.
 * @author Alejandro Gomez
 */
public interface Settings extends JDOMExternalizable {

    /**
     * Key for storing and retrieving the list of connections.
     */
    @NonNls
    String CONNECTIONS_KEY = "connections";
    /**
     * Key for storing and retrieving one connection.
     */
    @NonNls
    String CONNECTION_KEY = "connection";
    /**
     * Key for storing and retrieving the host.
     */
    @NonNls
    String HOST_KEY = "host";
    /**
     * Key for storing and retrieving the port.
     */
    @NonNls
    String PORT_KEY = "port";
    /**
     * Key for storing and retrieving the protocol.
     */
    @NonNls
    String PROTOCOL_KEY = "protocol";
    /**
     * Key for storing and retrieving the username.
     */
    @NonNls
    String USERNAME_KEY = "username";
    /**
     * Key for storing and retrieving the use password flag.
     */
    @NonNls
    String USE_PASSWORD_KEY = "usePassword";
    /**
     * Key for storing and retrieving the password.
     */
    @NonNls
    String PRIVATE_KEY_FILE_PATH_KEY = "privateKeyFilePath";
    /**
     * Key for storing and retrieving the private key file path.
     */
    @NonNls
    String PASSWORD_KEY = "password";
    /**
     * Key for storing and retrieving the passphrase.
     */
    @NonNls
    String PASSPHRASE_KEY = "passphrase";
    /**
     * Key for storing and retrieving the file path.
     */
    @NonNls
    String FILE_PATH_KEY = "filePath";
    /**
     * Key for storing and retrieving the alarm audio clip.
     */
    @NonNls
    String AUDIO_CLIP_PATH_KEY = "audioClipPath";
    /**
     * Key for storing and retrieving the buffer size.
     */
    @NonNls
    String BUFFER_SIZE_KEY = "bufferSize";
    /**
     * The key for retrieving the default alarm audio clip.
     */
    @NonNls
    String DEFAULT_AUDIO_CLIP_KEY = "remotetail.alarm.audioClip";
    /**
     * The default buffer size.
     */
    int DEFAULT_BUFFER_SIZE = 1000;

    /**
     * Returns the list of connections configured.
     * @return the list of connections configured.
     */
    List<ConnectionInfo> getConnections();

    /**
     * Returns the selected connection parameters.
     * @return the selected connection parameters.
     */
    ConnectionInfo getConnectionInfo();

    /**
     * Updates the buffer size and the alarm audio clip path.
     * @param _bufferSize the new buffer size.
     * @param _audioClipPath the new alarm audio clip path.
     */
    void update(int _bufferSize, String _audioClipPath);

    /**
     * Updates the parameters of the current selected connection.
     * @param _connectionInfo the new values for the current selected connection.
     */
    void updateConnectionInfo(@NotNull ConnectionInfo _connectionInfo);

    /**
     * Returns the alarm audio clip.
     * @return the alarm audio clip.
     */
    AudioClip getAudioClip();

    /**
     * Returns the alarm audio clip path.
     * @return the alarm audio clip path.
     */
    String getAudioClipPath();

    /**
     * Sets the new value for the alarm audio clip.
     * @param _audioClipPath the path for the new alarm audio clip.
     */
    void setAudioClipPath(String _audioClipPath);

    /**
     * Returns the buffer size.
     * @return the buffer size.
     */
    int getBufferSize();

    /**
     * Sets the new value for the buffer size.
     * @param _bufferSize the new value for the buffer size.
     */
    void setBufferSize(int _bufferSize);

    /**
     * Returns the index of the selected connection.
     * @return the index of the selected connection.
     */
    int getSelectedIndex();

    /**
     * Changes the actual selected connection.
     * @param _index the index of the new selected connection.
     */
    void setSelectedIndex(int _index);

    /**
     * Adds a new blank connection.
     */
    void add();

    /**
     * Removes a connection.
     * @param _index the index of the connection to remove.
     */
    void remove(int _index);

    /**
     * Duplicates a connection.
     * @param _index the index of the connection to duplicate.
     */
    void copy(int _index);

    /**
     * Checks if the configuration is modified.
     * @return true if the configuration is modified.
     */
    boolean isModified();

    /**
     * Applies the changes.
     */
    void apply();

    /**
     * Resets the changes.
     */
    void reset();
}
