/*
 *    Copyright 2006 Alejandro Gomez
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 *
 */
package ar.amgomez.idea.remotetail.settings;

import ar.amgomez.idea.remotetail.RemoteTailBundle;
import ar.amgomez.idea.remotetail.net.Protocol;
import ar.amgomez.idea.util.GenericCollectionUtils;
import ar.amgomez.idea.util.GenericTransformer;
import ar.amgomez.idea.util.StringUtils;
import com.intellij.openapi.diagnostic.Logger;
import com.intellij.openapi.util.InvalidDataException;
import com.intellij.openapi.util.JDOMExternalizer;
import com.intellij.openapi.util.PasswordUtil;
import com.intellij.openapi.util.WriteExternalException;
import com.intellij.openapi.util.text.StringUtil;
import org.jdom.Element;
import org.jetbrains.annotations.NonNls;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.applet.Applet;
import java.applet.AudioClip;
import java.io.File;
import java.net.MalformedURLException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

/**
 * Implementation for the definition and modification of the plugin configuration.
 * @author Alejandro Gomez
 */
public class SettingsImpl implements Settings {

    /**
     * Field LOGGER.
     * To inform application errors using IDEA's logging mechanism.
     */
    private static final Logger LOGGER = Logger.getInstance(SettingsImpl.class.getName());

    /**
     * The list of connections.
     */
    private List<ConnectionInfo> connections;
    /**
     * The index of the current selected connection.
     */
    private int index;
    /**
     * The alarm audio clip.
     */
    private AudioClip audioClip;
    /**
     * The alarm audio clip path.
     */
    @NonNls
    private String audioClipPath;
    /**
     * The buffer size.
     */
    private int bufferSize;
    /**
     * Keeps the settings before starting modifications to undo them later if required.
     */
    private SettingsMemento memento;

    /**
     * Creates a new <code>SettingsImpl<code>.
     */
    public SettingsImpl() {
        connections = new ArrayList<ConnectionInfo>();
        connections.add(new ConnectionInfo());
        audioClip = Applet.newAudioClip(RemoteTailBundle.resource(DEFAULT_AUDIO_CLIP_KEY));
        bufferSize = DEFAULT_BUFFER_SIZE;
        memento = new SettingsMemento();
    }

    /**
     * Returns the list of connections configured.
     * @return the list of connections configured.
     */
    public List<ConnectionInfo> getConnections() {
        return Collections.unmodifiableList(connections);
    }

    /**
     * Returns the selected connection parameters.
     * @return the selected connection parameters.
     */
    @Nullable
    public ConnectionInfo getConnectionInfo() {
        return index >= 0 ? connections.get(index) : null;
    }

    /**
     * Updates the buffer size and the alarm audio clip path.
     * @param _bufferSize the new buffer size.
     * @param _audioClipPath the new alarm audio clip path.
     */
    public void update(final int _bufferSize, final String _audioClipPath) {
        if (_bufferSize > 0) {
            bufferSize = _bufferSize;
        }
        audioClipPath = _audioClipPath;
    }

    /**
     * Updates the parameters of the current selected connection.
     * Notifies the listeners about the change.
     * @param _connectionInfo the new values for the current selected connection.
     */
    public void updateConnectionInfo(@NotNull final ConnectionInfo _connectionInfo) {
        if (index >= 0) {
            connections.set(index,  _connectionInfo);
        }
    }

    /**
     * Returns the alarm audio clip.
     * @return the alarm audio clip.
     */
    public AudioClip getAudioClip() {
        return audioClip;
    }

    /**
     * Returns the alarm audio clip path.
     * @return the alarm audio clip path.
     */
    public String getAudioClipPath() {
        return audioClipPath;
    }

    /**
     * Sets the new value for the alarm audio clip and loads the audio clip.
     * @param _audioClipPath the path for the new alarm audio clip.
     */
    public void setAudioClipPath(final String _audioClipPath) {
        audioClipPath = _audioClipPath;
        if (StringUtil.isEmpty(audioClipPath)) {
            audioClip = Applet.newAudioClip(RemoteTailBundle.resource(DEFAULT_AUDIO_CLIP_KEY));
        } else {
            try {
                audioClip = Applet.newAudioClip(new File(audioClipPath).toURL());
            } catch (MalformedURLException ex) {
                LOGGER.error("exception loading " + audioClipPath, ex);
            }
        }
    }

    /**
     * Returns the buffer size.
     * @return the buffer size.
     */
    public int getBufferSize() {
        return bufferSize;
    }

    /**
     * Sets the new value for the buffer size.
     * @param _bufferSize the new value for the buffer size.
     */
    public void setBufferSize(final int _bufferSize) {
        bufferSize = _bufferSize;
    }

    /**
     * Returns the index of the selected connection.
     * @return the index of the selected connection.
     */
    public int getSelectedIndex() {
        return index;
    }

    /**
     * Changes the actual selected connection.
     * @param _index the index of the new selected connection.
     */
    public void setSelectedIndex(final int _index) {
        index = _index;
    }

    /**
     * Adds a new blank connection.
     */
    public void add() {
        add(new ConnectionInfo());
    }

    /**
     * Removes a connection.
     * @param _index the index of the connection to remove.
     */
    public void remove(final int _index) {
        connections.remove(_index);
        if (_index < connections.size()) {
            setSelectedIndex(_index);
        } else if (connections.size() >= 0) {
            setSelectedIndex(_index - 1);
        } else {
            setSelectedIndex(-1);
        }
    }

    /**
     * Duplicates a connection.
     * @param _index the index of the connection to duplicate.
     */
    public void copy(final int _index) {
        add(new ConnectionInfo(connections.get(_index)));
    }

    /**
     * Adds a connection to the list.
     * Notifies the listeners about the change.
     * Selects the new connection.
     * @param _connectionInfo the connection to add.
     */
    private void add(final ConnectionInfo _connectionInfo) {
        connections.add(_connectionInfo);
        setSelectedIndex(connections.size() - 1);
    }

    /**
     * Checks if the configuration is modified.
     * @return true if the configuration is modified.
     */
    public boolean isModified() {
        return memento.isModified();
    }

    /**
     * Applies the changes.
     */
    public void apply() {
        memento = new SettingsMemento();
    }

    /**
     * Resets the changes.
     */
    public void reset() {
        memento.reset();
    }

    /**
     * Reads the stored settings.
     * @param _element the element containing the settings data.
     * @throws InvalidDataException
     */
    public void readExternal(final Element _element) throws InvalidDataException {
        connections.clear();
        final Element connectionsElement = _element.getChild(CONNECTIONS_KEY);
        if (connectionsElement != null) {
            final List children = connectionsElement.getChildren();
            for (final Object item : children) {
                final Element connectionElement = (Element)item;
                final ConnectionInfo connection = new ConnectionInfo();
                connection.setHost(connectionElement.getAttributeValue(HOST_KEY));
                connection.setPort(Integer.parseInt(connectionElement.getAttributeValue(PORT_KEY)));
                final String protocolValue = connectionElement.getAttributeValue(PROTOCOL_KEY);
                connection.setProtocol(protocolValue == null ? Protocol.SSH : Protocol.valueOf(protocolValue));
                connection.setFilePath(connectionElement.getAttributeValue(FILE_PATH_KEY));
                connection.setUsername(connectionElement.getAttributeValue(USERNAME_KEY));
                connection.setUsePassword(Boolean.parseBoolean(connectionElement.getAttributeValue(USE_PASSWORD_KEY)));
                connection.setPassword(PasswordUtil.decodePassword(connectionElement.getAttributeValue(PASSWORD_KEY)));
                connection.setPrivateKeyFilePath(connectionElement.getAttributeValue(PRIVATE_KEY_FILE_PATH_KEY));
                connection.setPassphrase(PasswordUtil.decodePassword(connectionElement.getAttributeValue(PASSPHRASE_KEY)));
                connections.add(connection);
            }
        }
        if (index == -1 && !connections.isEmpty()) {
            index = 0;
        }
        bufferSize = JDOMExternalizer.readInteger(_element, BUFFER_SIZE_KEY, DEFAULT_BUFFER_SIZE);
        setAudioClipPath(JDOMExternalizer.readString(_element, AUDIO_CLIP_PATH_KEY));
        memento = new SettingsMemento();
    }

    /**
     * Writes the settings.
     * @param _element the element where to add the settings data.
     * @throws WriteExternalException
     */
    public void writeExternal(final Element _element) throws WriteExternalException {
        final Element connectionsElement = new Element(CONNECTIONS_KEY);
        for (final ConnectionInfo info : connections) {
            final Element connectionElement = new Element(CONNECTION_KEY);
            connectionElement.setAttribute(HOST_KEY, StringUtils.emptyIfNull(info.getHost()));
            connectionElement.setAttribute(PORT_KEY, String.valueOf(info.getPort()));
            connectionElement.setAttribute(PROTOCOL_KEY, info.getProtocol().name());
            connectionElement.setAttribute(FILE_PATH_KEY, StringUtils.emptyIfNull(info.getFilePath()));
            connectionElement.setAttribute(USERNAME_KEY, StringUtils.emptyIfNull(info.getUsername()));
            connectionElement.setAttribute(USE_PASSWORD_KEY, String.valueOf(info.isUsePassword()));
            connectionElement.setAttribute(PASSWORD_KEY, StringUtils.emptyIfNull(PasswordUtil.encodePassword(info.getPassword())));
            connectionElement.setAttribute(PRIVATE_KEY_FILE_PATH_KEY, StringUtils.emptyIfNull(info.getPrivateKeyFilePath()));
            connectionElement.setAttribute(PASSPHRASE_KEY, StringUtils.emptyIfNull(PasswordUtil.encodePassword(info.getPassphrase())));
            connectionsElement.addContent(connectionElement);
        }
        _element.addContent(connectionsElement);
        JDOMExternalizer.write(_element, AUDIO_CLIP_PATH_KEY, StringUtils.emptyIfNull(audioClipPath));
        JDOMExternalizer.write(_element, BUFFER_SIZE_KEY, bufferSize);
    }

    /**
     * Keeps the configuration values before the modifications start so they can be undone later.
     * @author Alejandro Gomez
     */
    private class SettingsMemento {

        /**
         * The list of connections.
         */
        private List<ConnectionInfo> origConnections;
        /**
         * The index of the selected connection.
         */
        private int origIndex;
        /**
         * The alarm audio clip path.
         */
        @NonNls
        private String origAudioClipPath;
        /**
         * The buffer size.
         */
        private int origBufferSize;

        /**
         * Creates a new <code>SettingsMemento</code> copying the actual configuration values.
         */
        private SettingsMemento() {
            origConnections = GenericCollectionUtils.collect(connections, new GenericTransformer<ConnectionInfo, ConnectionInfo>() {
                public ConnectionInfo transform(final ConnectionInfo _object) {
                    return new ConnectionInfo(_object);
                }
            });
            origIndex = index;
            origAudioClipPath = audioClipPath;
            origBufferSize = bufferSize;
        }

        /**
         * Resets the configuration values with the stored ones.
         */
        public void reset() {
            connections = new ArrayList<ConnectionInfo>(origConnections);
            index = origIndex;
            setAudioClipPath(origAudioClipPath);
            bufferSize = origBufferSize;
        }

        /**
         * Checks if the configuration values changed.
         * @return true if the configuration values changed.
         */
        public boolean isModified() {
            return !origConnections.equals(connections) ||
                    StringUtil.isEmpty(origAudioClipPath) && !StringUtil.isEmpty(audioClipPath) ||
                    !StringUtil.isEmpty(origAudioClipPath) && !origAudioClipPath.equals(audioClipPath) ||
                    origBufferSize != bufferSize;
        }
    }
}
