/*
 *    Copyright 2006 Alejandro Gomez
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 *
 */
package ar.amgomez.idea.remotetail.ui;

import ar.amgomez.idea.remotetail.RemoteTailBundle;
import com.intellij.openapi.diagnostic.Logger;
import org.jetbrains.annotations.NonNls;

import javax.swing.BorderFactory;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JWindow;
import javax.swing.SwingConstants;
import javax.swing.border.Border;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Font;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.IOException;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Properties;

/**
 * The about window.
 * @author Alejandro Gomez
 */
public class AboutWindow extends JWindow {

    /**
     * To inform application errors using IDEA's logging mechanism.
     */
    private static final Logger LOGGER = Logger.getInstance(AboutWindow.class.getName());

    /**
     * The singleton instance.
     */
    private static final AboutWindow INSTANCE = new AboutWindow();

    /**
     * The name of the font to use in the labels.
     */
    @NonNls
    private static final String LABEL_FONT_NAME = "Sans-Serif";
    /**
     * The font to use in the labels.
     */
    private static final Font LABEL_FONT = new Font(LABEL_FONT_NAME, Font.PLAIN, 10);
    /**
     * The build.properties file path.
     */
    @NonNls
    private static final String BUILD_PROPERTIES_PATH = "/ar/amgomez/idea/remotetail/build.properties";
    /**
     * The pattern to parse the build date.
     */
    @NonNls
    private static final String DATE_PATTERN = "yyyyMMdd";

    /**
     * The version's major number.
     */
    private String versionMajor;
    /**
     * The version's minor number.
     */
    private String versionMinor;
    /**
     * The version's revision number.
     */
    private String versionRevision;
    /**
     * The build number.
     */
    private String buildNumber;
    /**
     * The build date.
     */
    private Date buildDate;

    /**
     * Constructs a new <code>AboutWindow</code>.
     * Reads the build information and creates the UI components.
     * It is private to prevent other instances.
     */
    private AboutWindow() {
        final Properties buildProperties = new Properties();
        try {
            buildProperties.load(AboutWindow.class.getResourceAsStream(BUILD_PROPERTIES_PATH));
        } catch (IOException ex) {
            throw new ExceptionInInitializerError(ex);
        }
        versionMajor = buildProperties.getProperty("remotetail.version.major");
        versionMinor = buildProperties.getProperty("remotetail.version.minor");
        versionRevision = buildProperties.getProperty("remotetail.version.revision");
        buildNumber = buildProperties.getProperty("remotetail.buildNumber");
        final String buildDateStr = buildProperties.getProperty("remotetail.buildDate");
        final DateFormat dateFormat = new SimpleDateFormat(DATE_PATTERN);
        try {
            buildDate = dateFormat.parse(buildDateStr);
        } catch (ParseException ex) {
            LOGGER.info("exception parsing " + buildDateStr, ex);
            buildDate = new Date();
        }
        build();
    }

    /**
     * Creates the UI components.
     */
    private void build() {
        final JPanel contentPane = (JPanel)getContentPane();
        contentPane.setLayout(new BorderLayout());
        contentPane.setBorder(BorderFactory.createLineBorder(Color.BLACK, 1));
        contentPane.add(getSplashLabel(), BorderLayout.CENTER);
        contentPane.add(getSouthPanel(), BorderLayout.SOUTH);
        addMouseListener(new MouseAdapter() {
            @Override
            public void mouseClicked(final MouseEvent _event) {
                setVisible(false);
            }
        });
        addKeyListener(new KeyAdapter() {
            @Override
            public void keyPressed(final KeyEvent _event) {
                if (_event.getKeyCode() == KeyEvent.VK_ESCAPE) {
                    setVisible(false);
                }
            }
        });
        pack();
    }

    /**
     * Returns the label with the image.
     * @return the label with the image.
     */
    private static JLabel getSplashLabel() {
        return new JLabel(RemoteTailBundle.icon("remotetail.about.image"));
    }

    /**
     * Returns the panel with the build information.
     * @return the panel with the build information.
     */
    private JComponent getSouthPanel() {
        final JPanel southPanel = new JPanel(new BorderLayout());
        southPanel.setBackground(Color.WHITE);
        southPanel.add(getVersionLabel(), BorderLayout.NORTH);
        southPanel.add(getCopyrightLabel(), BorderLayout.SOUTH);
        return southPanel;
    }

    /**
     * Returns the label with the copyright information.
     * @return the label with the copyright information.
     */
    private static JLabel getCopyrightLabel() {
        final JLabel copyrightLabel = new JLabel(getCopyright(), SwingConstants.TRAILING);
        copyrightLabel.setBorder(getLabelBorder());
        copyrightLabel.setFont(LABEL_FONT);
        return copyrightLabel;
    }

    /**
     * Returns the label with the version information.
     * @return the label with the version information.
     */
    private JLabel getVersionLabel() {
        final JLabel versionLabel = new JLabel(getVersion(), SwingConstants.TRAILING);
        versionLabel.setBorder(getLabelBorder());
        versionLabel.setFont(LABEL_FONT);
        return versionLabel;
    }

    /**
     * Creates a border for the labels.
     * @return returns the border for the label.
     */
    private static Border getLabelBorder() {
        return BorderFactory.createEmptyBorder(0, 0, 5, 5);
    }

    /**
     * Returns the copyright text.
     * @return the copyright text.
     */
    private static String getCopyright() {
        return RemoteTailBundle.message("remotetail.about.copyright");
    }

    /**
     * Returns the version text.
     * @return the version text.
     */
    private String getVersion() {
        return RemoteTailBundle.message("remotetail.about.version", versionMajor, versionMinor, versionRevision, buildNumber, buildDate);
    }

    /**
     * Returns the singleton instance.
     * @return the singleton instance.
     */
    public static AboutWindow getInstance() {
        return INSTANCE;
    }
}
