/*
 *    Copyright 2006 Alejandro Gomez
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 *
 */
package ar.amgomez.idea.remotetail.ui;

import ar.amgomez.idea.remotetail.LogConsoleModel;
import ar.amgomez.idea.remotetail.RemoteTailBundle;
import ar.amgomez.idea.remotetail.actions.ErrorAlarmAction;
import ar.amgomez.idea.remotetail.actions.ErrorPopupAction;
import ar.amgomez.idea.remotetail.actions.FilterErrorAction;
import ar.amgomez.idea.remotetail.actions.FilterInfoAction;
import ar.amgomez.idea.remotetail.actions.FilterWarningAction;
import ar.amgomez.idea.remotetail.actions.LevelComboBoxAction;
import ar.amgomez.idea.remotetail.events.LogEntryListener;
import ar.amgomez.idea.remotetail.events.LogHeaderListener;
import ar.amgomez.idea.remotetail.logging.LogEntry;
import com.intellij.openapi.actionSystem.ActionManager;
import com.intellij.openapi.actionSystem.DefaultActionGroup;
import com.intellij.openapi.application.ApplicationManager;
import com.intellij.openapi.ui.Messages;

import javax.swing.JPanel;
import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Container;

/**
 * Base abstract <code>LogConsoleView</code> implementation.
 * @author Alejandro Gomez
 */
public abstract class AbstractLogConsoleView implements LogConsoleView {

    /**
     * The model associated with this <code>LogConsoleView</code>.
     */
    private LogConsoleModel model;
    /**
     * The UI component.
     */
    private Container panel;

    /**
     * Constructs a new <code>AbstractLogConsoleView</code>.
     * @param _model the model associated with the new instance.
     */
    protected AbstractLogConsoleView(final LogConsoleModel _model) {
        model = _model;
        model.addLogEntryListener(new LogEntryListener() {
            public void unfilteredLogEntryAdded(final LogEntry _logEntry) {
                printLogEntry(_logEntry);
            }

            public void logFilterChanged() {
                clear();
                for (final LogEntry logEntry : model.getUnfilteredLogEntries()) {
                    printLogEntry(logEntry);
                }
            }

            public void logProviderError(final String _message) {
                final String message = RemoteTailBundle.message("remotetail.tailError.message", model.getFilePath(), _message);
                ApplicationManager.getApplication().invokeLater(new Runnable() {
                    public void run() {
                        Messages.showErrorDialog(message, RemoteTailBundle.message("remotetail.tailError.title"));
                    }
                });
            }

            public void logProviderException(final Exception _exception) {
                final String message = RemoteTailBundle.message("remotetail.tailException.message", model.getFilePath(), _exception.getMessage());
                ApplicationManager.getApplication().invokeLater(new Runnable() {
                    public void run() {
                        Messages.showErrorDialog(message, RemoteTailBundle.message("remotetail.tailException.title"));
                    }
                });
            }
        });
    }

    /**
     * Prints a <code>LogEntry</code>.
     * @param _logEntry the <code>LogEntry</code> to print.
     */
    protected abstract void printLogEntry(LogEntry _logEntry);

    /**
     * Clears the console.
     */
    protected abstract void clear();

    /**
     * Returns the model associated with this console.
     * @return the model associated with this console.
     */
    public LogConsoleModel getModel() {
        return model;
    }

    /**
     * Returns the title.
     * @return the title.
     */
    public String getTitle() {
        return model.getTitle();
    }

    /**
     * Initializes the UI components.
     * Builds the panel with the toolbar and the viewing component.
     */
    private void init() {
        panel = new JPanel(new BorderLayout());
        panel.add(getToolbar(), BorderLayout.NORTH);
        panel.add(getConsoleComponent(), BorderLayout.CENTER);
    }

    /**
     * Initializes, if necessary, and returns the UI component.
     * @return the UI component.
     */
    public Component getComponent() {
        if (panel == null) {
            init();
        }
        return panel;
    }

    /**
     * Returns the component for viewing the log.
     * @return the component for viewing the log.
     */
    protected abstract Component getConsoleComponent();

    /**
     * Builds and returns the toolbar with all the actions' buttons.
     * @return the toolbar.
     */
    private Component getToolbar() {
        final ActionManager manager = ActionManager.getInstance();
        final DefaultActionGroup actionGroup = new DefaultActionGroup();
        actionGroup.add(new FilterInfoAction(model));
        actionGroup.add(new FilterWarningAction(model));
        actionGroup.add(new FilterErrorAction(model));
        actionGroup.addSeparator();
        actionGroup.add(new ErrorPopupAction(model));
        actionGroup.add(new ErrorAlarmAction(model));
        actionGroup.add(new LevelComboBoxAction(model));
/*TODO
        actionGroup.addSeparator();
        actionGroup.add(new PrintAction(model));
*/
        return manager.createActionToolbar("RemoteTail.filterToolbar", actionGroup, true).getComponent();
    }

    /**
     * Registers a new <code>LogHeaderListener</code>.
     * @param _listener the <code>LogHeaderListener</code> to register.
     */
    public void addLogHeaderListener(final LogHeaderListener _listener) {
        model.addLogHeaderListener(_listener);
    }

    /**
     * Unregisters a <code>LogHeaderListener</code>.
     * @param _listener the <code>LogHeaderListener</code> to unregister.
     */
    public void removeLogHeaderListener(final LogHeaderListener _listener) {
        model.removeLogHeaderListener(_listener);
    }
}
