/*
 *    Copyright 2006 Alejandro Gomez
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 *
 */
package ar.amgomez.idea.remotetail.ui;

import ar.amgomez.idea.remotetail.RemoteTailBundle;
import ar.amgomez.idea.remotetail.net.Client;
import ar.amgomez.idea.remotetail.net.ClientFactory;
import ar.amgomez.idea.remotetail.net.Protocol;
import ar.amgomez.idea.remotetail.net.SSHClient;
import ar.amgomez.idea.remotetail.settings.ConnectionInfo;
import ar.amgomez.idea.remotetail.settings.Settings;
import ar.amgomez.idea.util.IdeaVersionsFacade;
import com.intellij.openapi.actionSystem.ActionManager;
import com.intellij.openapi.actionSystem.AnAction;
import com.intellij.openapi.actionSystem.AnActionEvent;
import com.intellij.openapi.actionSystem.DefaultActionGroup;
import com.intellij.openapi.diagnostic.Logger;
import com.intellij.openapi.fileChooser.FileChooserDescriptor;
import com.intellij.openapi.fileChooser.FileChooserDialog;
import com.intellij.openapi.fileChooser.FileChooserFactory;
import com.intellij.openapi.progress.ProgressIndicator;
import com.intellij.openapi.progress.ProgressManager;
import com.intellij.openapi.ui.Messages;
import com.intellij.openapi.ui.TextFieldWithBrowseButton;
import com.intellij.openapi.util.text.StringUtil;
import com.intellij.openapi.vfs.VfsUtil;
import com.intellij.openapi.vfs.VirtualFile;
import com.intellij.ui.DocumentAdapter;

import javax.swing.*;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.IOException;
import java.net.MalformedURLException;
import java.util.ArrayList;
import java.util.List;

/**
 * Class ConnectionForm.
 * @author Alejandro Gomez
 */
public class ConnectionForm {

    private static final Logger LOGGER = Logger.getInstance(ConnectionForm.class.getName());

    /**
     * Field settings.
     */
    private Settings settings;
    /**
     * Field list.
     */
    private JList list;
    /**
     * Field txtHost.
     */
    private JTextField txtHost;
    /**
     * Field txtPort.
     */
    private JTextField txtPort;
    private JRadioButton btnSSHProtocol;
    private JRadioButton btnTelnetProtocol;
    /**
     * Field txtFilePath.
     */
    private TextFieldWithBrowseButton txtFilePath;
    /**
     * Field panel.
     */
    private JPanel panel;
    /**
     * Field toolbarPanel.
     */
    private JPanel toolbarPanel;
    private JRadioButton btnPasswordAuth;
    private JRadioButton btnPrivateKeyAuth;
    private JTextField txtUsername;
    private JTextField txtPassword;
    private TextFieldWithBrowseButton txtPrivateKeyFilePath;
    private JTextField txtPassphrase;
    private DocumentListener docListener;
    private ActionListener passListener;
    private ActionListener sshListener;
    private List<ListSelectionListener> listeners;

    /**
     * Class ConnectionForm Constructor.
     * @param _settings
     */
    public ConnectionForm(final Settings _settings) {
        settings = _settings;
        listeners = new ArrayList<ListSelectionListener>();
        list.setModel(new SettingsListModel());
        list.setCellRenderer(new ConnectionInfoListCellRenderer());
        txtPort.setInputVerifier(new NumericInputVerifier());
        list.getSelectionModel().addListSelectionListener(new ListSelectionListener() {
            public void valueChanged(final ListSelectionEvent _event) {
                if (!_event.getValueIsAdjusting()) {
                    updateInputFields();
                }
            }
        });
        passListener = new ActionListener() {
            public void actionPerformed(final ActionEvent _event) {
                setPasswordEnabled(btnPasswordAuth.isSelected());
            }
        };
        btnPasswordAuth.addActionListener(passListener);
        btnPrivateKeyAuth.addActionListener(passListener);
        sshListener = new ActionListener() {
            public void actionPerformed(final ActionEvent _event) {
                setSSHProtocolEnabled(btnSSHProtocol.isSelected());
            }
        };
        btnSSHProtocol.addActionListener(sshListener);
        btnTelnetProtocol.addActionListener(sshListener);
        list.setSelectedIndex(settings.getSelectedIndex());
        buildToolbar();
        docListener = new DocumentAdapter() {
            @Override
            protected void textChanged(final DocumentEvent _event) {
                updateConnectionInfo();
            }
        };
        registerChangeListeners();
    }

    private void setPasswordEnabled(final boolean _enabled) {
        unregisterChangeListeners();
        txtPassword.setEnabled(_enabled);
        txtPrivateKeyFilePath.setEnabled(!_enabled);
        txtPassphrase.setEnabled(!_enabled);
        registerChangeListeners();
        updateConnectionInfo();
    }

    private void setSSHProtocolEnabled(final boolean _enabled) {
        unregisterChangeListeners();
        if (!_enabled) {
            btnPasswordAuth.setSelected(true);
        }
        btnPrivateKeyAuth.setEnabled(_enabled);
        registerChangeListeners();
        if (!_enabled) {
            setPasswordEnabled(true);
        } else {
            updateConnectionInfo();
        }
    }

    private void updateConnectionInfo() {
        int port;
        try {
            port = Integer.parseInt(txtPort.getText());
        } catch (NumberFormatException ex) {
            port = 0;
        }
        final ConnectionInfo connectionInfo = new ConnectionInfo();
        connectionInfo.setHost(txtHost.getText());
        connectionInfo.setPort(port);
        connectionInfo.setHost(txtHost.getText());
        connectionInfo.setProtocol(btnTelnetProtocol.isSelected() ? Protocol.TELNET : Protocol.SSH);
        connectionInfo.setFilePath(txtFilePath.getText());
        connectionInfo.setUsername(txtUsername.getText());
        connectionInfo.setUsePassword(btnPasswordAuth.isSelected());
        connectionInfo.setPassword(txtPassword.getText());
        connectionInfo.setPrivateKeyFilePath(txtPrivateKeyFilePath.getText());
        connectionInfo.setPassphrase(txtPassphrase.getText());
        settings.updateConnectionInfo(connectionInfo);
        txtFilePath.setButtonEnabled(settings.getConnectionInfo().canNavigate());
        fireConnectionChanged();
    }

    private void updateInputFields() {
        final int index = list.getSelectedIndex();
        unregisterChangeListeners();
        if (index >= 0) {
            settings.setSelectedIndex(index);
            final ConnectionInfo connectionInfo = settings.getConnectionInfo();
            txtHost.setText(connectionInfo.getHost());
            txtPort.setText(String.valueOf(connectionInfo.getProtocol().getDefaultPort()));
            txtFilePath.setText(connectionInfo.getFilePath());
            txtFilePath.setButtonEnabled(connectionInfo.canNavigate());
            txtUsername.setText(connectionInfo.getUsername());
            txtPassword.setText(connectionInfo.getPassword());
            txtPrivateKeyFilePath.setText(connectionInfo.getPrivateKeyFilePath());
            txtPassphrase.setText(connectionInfo.getPassphrase());
            if (connectionInfo.getProtocol() == Protocol.TELNET) {
                btnTelnetProtocol.setSelected(true);
                btnPrivateKeyAuth.setEnabled(false);
            } else {
                btnSSHProtocol.setSelected(true);
                btnPrivateKeyAuth.setEnabled(true);
            }
            if (connectionInfo.isUsePassword()) {
                btnPasswordAuth.setSelected(true);
                txtPassword.setEnabled(true);
                txtPrivateKeyFilePath.setEnabled(false);
                txtPassphrase.setEnabled(false);
            } else {
                btnPrivateKeyAuth.setSelected(true);
                txtPassword.setEnabled(false);
                txtPrivateKeyFilePath.setEnabled(true);
                txtPassphrase.setEnabled(true);
            }
            fireConnectionChanged();
        } else {
            txtHost.setText("");
            txtPort.setText(String.valueOf(Protocol.SSH.getDefaultPort()));
            txtFilePath.setText("");
            txtFilePath.setButtonEnabled(false);
            txtUsername.setText("");
            txtPassword.setText("");
            txtPrivateKeyFilePath.setText("");
            txtPassphrase.setText("");
            btnPasswordAuth.setSelected(true);
            btnSSHProtocol.setSelected(true);
        }
        registerChangeListeners();
    }

    /**
     * Method buildToolbar.
     */
    private void buildToolbar() {
        final DefaultActionGroup actionGroup = new DefaultActionGroup();
        final ActionManager manager = ActionManager.getInstance();
        actionGroup.add(new AddLogConnectionAction());
        actionGroup.add(new CopyLogConnectionAction());
        actionGroup.add(new RemoveLogConnectionAction());
        toolbarPanel.add(manager.createActionToolbar("RemoteTail.logFile.toolbar", actionGroup, true).getComponent());
    }

    /**
     * returns the value for panel.
     * @return JPanel
     */
    public JPanel getPanel() {
        return panel;
    }

    private void registerChangeListeners() {
        txtHost.getDocument().addDocumentListener(docListener);
        txtPort.getDocument().addDocumentListener(docListener);
        txtUsername.getDocument().addDocumentListener(docListener);
        txtFilePath.getTextField().getDocument().addDocumentListener(docListener);
        txtPassword.getDocument().addDocumentListener(docListener);
        txtPrivateKeyFilePath.getTextField().getDocument().addDocumentListener(docListener);
        txtPassphrase.getDocument().addDocumentListener(docListener);
        btnPasswordAuth.addActionListener(passListener);
        btnPrivateKeyAuth.addActionListener(passListener);
        btnSSHProtocol.addActionListener(sshListener);
        btnTelnetProtocol.addActionListener(sshListener);
    }

    private void unregisterChangeListeners() {
        txtHost.getDocument().removeDocumentListener(docListener);
        txtPort.getDocument().removeDocumentListener(docListener);
        txtUsername.getDocument().removeDocumentListener(docListener);
        txtFilePath.getTextField().getDocument().removeDocumentListener(docListener);
        txtPassword.getDocument().removeDocumentListener(docListener);
        txtPrivateKeyFilePath.getTextField().getDocument().removeDocumentListener(docListener);
        txtPassphrase.getDocument().removeDocumentListener(docListener);
        btnPasswordAuth.removeActionListener(passListener);
        btnPrivateKeyAuth.removeActionListener(passListener);
        btnSSHProtocol.removeActionListener(sshListener);
        btnTelnetProtocol.removeActionListener(sshListener);
    }

    private void createUIComponents() {
        txtFilePath = new TextFieldWithBrowseButton(new ActionListener() {
            public void actionPerformed(final ActionEvent _event) {
                final ConnectionInfo connectionInfo = settings.getConnectionInfo();
                final Client client = ClientFactory.getInstance().getClient(connectionInfo);
                final boolean[] result = {false};
                final Exception[] exception = {null};
                IdeaVersionsFacade.runProcessWithProgressSynchronously(new Runnable() {
                    public void run() {
                        final ProgressIndicator indicator = ProgressManager.getInstance().getProgressIndicator();
                        indicator.setIndeterminate(true);
                        indicator.setText(RemoteTailBundle.message("remotetail.connection.progress.text", connectionInfo.getHost(), connectionInfo.getUsername()));
                        try {
                            result[0] = client.connect() && client.authenticate();
                        } catch (IOException ex) {
                            exception[0] = ex;
                        }
                    }
                }, RemoteTailBundle.message("remotetail.connection.progress.title"), false, null);
                try {
                    if (result[0]) {
                        final RemoteFileChooser remoteFileChooser = new RemoteFileChooser((SSHClient)client);
                        final RemoteFileChooserDialog dialog = new RemoteFileChooserDialog(remoteFileChooser, ConnectionForm.this.getPanel());
                        dialog.show();
                        if (dialog.isOK()) {
                            txtFilePath.setText(remoteFileChooser.getSelectedFilePath());
                            updateConnectionInfo();
                        }
                    } else {
                        final String title;
                        final String message;
                        if (exception[0] != null) {
                            LOGGER.info(exception[0]);
                            title = RemoteTailBundle.message("remotetail.connectException.title");
                            message = RemoteTailBundle.message("remotetail.connectException.message", connectionInfo.getHost(), exception[0].getMessage());
                        } else {
                            title = RemoteTailBundle.message("remotetail.connectError.title");
                            message = RemoteTailBundle.message("remotetail.connectError.message", connectionInfo.getHost());
                        }
                        Messages.showErrorDialog(message, title);
                    }
                } catch (IOException ex) {
                    LOGGER.info(ex);
                    Messages.showErrorDialog(RemoteTailBundle.message("remotetail.connectException.message", connectionInfo.getHost(), ex.getMessage()), RemoteTailBundle.message("remotetail.connectException.title"));
                } finally {
                    try {
                        client.disconnect();
                    } catch (IOException ex) {
                        LOGGER.error(ex);
                    }
                }
            }
        });
        txtPrivateKeyFilePath = new TextFieldWithBrowseButton(new ActionListener() {
            public void actionPerformed(final ActionEvent _event) {
                VirtualFile file;
                if (StringUtil.isEmpty(txtPrivateKeyFilePath.getText())) {
                    file = getHomeFolder();
                } else {
                    try {
                        file = VfsUtil.findFileByURL(new File(txtPrivateKeyFilePath.getText()).toURL());
                    } catch (Exception ex) {
                        file = getHomeFolder();
                    }
                }
                final FileChooserDialog fileChooser = FileChooserFactory.getInstance().createFileChooser(new FileChooserDescriptor(true, false, false, false, false, false), panel);
                final VirtualFile[] files = fileChooser.choose(file, null);
                if (files.length > 0) {
                    txtPrivateKeyFilePath.setText(files[0].getPath());
                    updateConnectionInfo();
                }
            }

            private VirtualFile getHomeFolder() {
                try {
                    return VfsUtil.findFileByURL(new File(System.getProperty("user.home")).toURL());
                } catch (MalformedURLException ex) {
                    LOGGER.error(ex);
                    return null;
                }
            }
        });
    }

    public void dispose() {
        unregisterChangeListeners();
        listeners.clear();
    }

    /**
     * Registers a new <code>ListSelectionListener</code>.
     *
     * @param _listener the <code>ListSelectionListener</code> to register.
     */
    public synchronized void addSettingsChangeListener(final ListSelectionListener _listener) {
        listeners.add(_listener);
    }

    /**
     * Unregisters a <code>ListSelectionListener</code>.
     *
     * @param _listener the <code>ListSelectionListener</code> to unregister.
     */
    public synchronized void removeSettingsChangeListener(final ListSelectionListener _listener) {
        listeners.remove(_listener);
    }

    /**
     * Notifies the listeners about the selection change.
     *
     */
    private synchronized void fireConnectionChanged() {
        final int index = settings.getSelectedIndex();
        final ListSelectionEvent event = new ListSelectionEvent(this, index, index, false);
        for (final ListSelectionListener lsListener : listeners) {
            lsListener.valueChanged(event);
        }
    }

    /**
     * Class AddLogConnectionAction.
     * @author Alejandro Gomez
     */
    private class AddLogConnectionAction extends AnAction {

        /**
         * Class AddLogConnectionAction Constructor.
         */
        private AddLogConnectionAction() {
            super(RemoteTailBundle.message("remotetail.settings.add.text"), RemoteTailBundle.message("remotetail.settings.add.description"), RemoteTailBundle.icon("remotetail.settings.add.icon"));
        }

        /**
         * Method update.
         * @param _event
         */
        @Override
        public void update(final AnActionEvent _event) {
            super.update(_event);
            _event.getPresentation().setEnabled(true);
        }

        /**
         * Method actionPerformed.
         * @param _event
         */
        @Override
        public void actionPerformed(final AnActionEvent _event) {
            settings.add();
            ((SettingsListModel)list.getModel()).add();
        }
    }

    /**
     * Class CopyLogConnectionAction.
     * @author Alejandro Gomez
     */
    private class CopyLogConnectionAction extends AnAction {

        /**
         * Class CopyLogConnectionAction Constructor.
         */
        private CopyLogConnectionAction() {
            super(RemoteTailBundle.message("remotetail.settings.copy.text"), RemoteTailBundle.message("remotetail.settings.copy.description"), RemoteTailBundle.icon("remotetail.settings.copy.icon"));
        }

        /**
         * Method update.
         * @param _event
         */
        @Override
        public void update(final AnActionEvent _event) {
            super.update(_event);
            _event.getPresentation().setEnabled(list.getSelectedIndex() != -1);
        }

        /**
         * Method actionPerformed.
         * @param _event
         */
        @Override
        public void actionPerformed(final AnActionEvent _event) {
            settings.copy(list.getSelectedIndex());
            ((SettingsListModel)list.getModel()).add();
        }
    }

    /**
     * Class RemoveLogConnectionAction.
     * @author Alejandro Gomez
     */
    private class RemoveLogConnectionAction extends AnAction {

        /**
         * Class RemoveLogConnectionAction Constructor.
         */
        private RemoveLogConnectionAction() {
            super(RemoteTailBundle.message("remotetail.settings.remove.text"), RemoteTailBundle.message("remotetail.settings.remove.description"), RemoteTailBundle.icon("remotetail.settings.remove.icon"));
        }

        /**
         * Method update.
         * @param _event
         */
        @Override
        public void update(final AnActionEvent _event) {
            super.update(_event);
            _event.getPresentation().setEnabled(list.getSelectedIndex() != -1);
        }

        /**
         * Method actionPerformed.
         * @param _event
         */
        @Override
        public void actionPerformed(final AnActionEvent _event) {
            final int index = list.getSelectedIndex();
            settings.remove(index);
            ((SettingsListModel)list.getModel()).remove(index);
        }
    }

    /**
     * Class SettingsListModel.
     * @author Alejandro Gomez
     */
    private class SettingsListModel extends AbstractListModel {

        /**
         * returns the value for size.
         * @return int
         */
        public int getSize() {
            return settings.getConnections().size();
        }

        /**
         * Method getElementAt.
         * @param _index
         * @return Object
         */
        public Object getElementAt(final int _index) {
            return settings.getConnections().get(_index);
        }

        private void add() {
            final int index = settings.getSelectedIndex();
            fireIntervalAdded(this, index, index);
            list.setSelectedIndex(index);
        }

        private void remove(final int _index) {
            fireIntervalRemoved(this, _index, _index);
            list.setSelectedIndex(settings.getSelectedIndex());
        }
    }

    /**
     * Class ConnectionInfoListCellRenderer.
     * @author Alejandro Gomez
     */
    private static class ConnectionInfoListCellRenderer extends DefaultListCellRenderer {

        /**
         * Method getListCellRendererComponent.
         * @param _list
         * @param _value
         * @param _index
         * @param _isSelected
         * @param _cellHasFocus
         * @return Component
         */
        @Override
        public Component getListCellRendererComponent(final JList _list, final Object _value, final int _index, final boolean _isSelected, final boolean _cellHasFocus) {
            super.getListCellRendererComponent(_list, _value, _index, _isSelected, _cellHasFocus);
            final ConnectionInfo connectionInfo = (ConnectionInfo)_value;
            setText(connectionInfo.getDescription() == null ? RemoteTailBundle.message("remotetail.settings.unnamedConnection") : connectionInfo.getDescription());
            return this;
        }
    }
}
