/*
 *    Copyright 2006 Alejandro Gomez
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 *
 */
package ar.amgomez.idea.remotetail.ui;

import ar.amgomez.idea.util.IdeaVersionsFacade;
import com.intellij.execution.filters.Filter;
import com.intellij.execution.filters.HyperlinkInfo;
import com.intellij.execution.process.ProcessHandler;
import com.intellij.execution.ui.ConsoleView;
import com.intellij.execution.ui.ConsoleViewContentType;
import com.intellij.openapi.project.Project;
import org.picocontainer.Disposable;

import javax.swing.JComponent;

/**
 * An implementation of <code>ConsoleView</code> that delegates all method calls.
 */
public class ConsoleViewWrapper implements ConsoleView, Disposable {

    /**
     * The wrapped <code>ConsoleView</code>.
     */
    private ConsoleView wrapped;

    /**
     * Constructs a new <code>ConsoleViewWrapper</code>.
     * @param _project the open project.
     */
    public ConsoleViewWrapper(final Project _project) {
        wrapped = IdeaVersionsFacade.getConsoleView(_project);
    }

    /**
     * Delegates the call to <code>print</code>.
     * @param _text the text to print
     * @param _contentType the type for using when printing the text.
     */
    public void print(final String _text, final ConsoleViewContentType _contentType) {
        wrapped.print(_text, _contentType);
    }

    /**
     * Delegates the call to <code>clear</code>.
     * Clears the console.
     */
    public void clear() {
        wrapped.clear();
    }

    /**
     * Delegates the call to <code>scrollTo</code>.
     * @param _offset the offset.
     */
    public void scrollTo(final int _offset) {
        wrapped.scrollTo(_offset);
    }

    /**
     * Delegates the call to <code>attachToProcess</code>.
     * @param _processHandler the <code>ProcessHandler</code>.
     */
    public void attachToProcess(final ProcessHandler _processHandler) {
        wrapped.attachToProcess(_processHandler);
    }

    /**
     * Delegates the call to <code>setOutputPaused</code>.
     * @param _value the new value for <code>outputPaused</code>.
     */
    public void setOutputPaused(final boolean _value) {
        wrapped.setOutputPaused(_value);
    }

    /**
     * Delegates the call to <code>isOutputPaused</code>.
     * @return the value of <code>outputPaused</code>.
     */
    public boolean isOutputPaused() {
        return wrapped.isOutputPaused();
    }

    /**
     * Delegates the call to <code>hasDeferredOutput</code>.
     * @return the value of <code>hasDeferredOutput</code>.
     */
    public boolean hasDeferredOutput() {
        return wrapped.hasDeferredOutput();
    }

    /**
     * Delegates the call to <code>performWhenNoDeferredOutput</code>.
     * @param _runnable the process to execute.
     */
    public void performWhenNoDeferredOutput(final Runnable _runnable) {
        wrapped.performWhenNoDeferredOutput(_runnable);
    }

    /**
     * Delegates the call to <code>setHelpId</code>.
     * @param _helpId the new value for helpId.
     */
    public void setHelpId(final String _helpId) {
        wrapped.setHelpId(_helpId);
    }

    /**
     * Delegates the call to <code>addMessageFilter</code>.
     * @param _filter the <code>Filter</code>.
     */
    public void addMessageFilter(final Filter _filter) {
        wrapped.addMessageFilter(_filter);
    }

    /**
     * Delegates the call to <code>printHyperlink</code>.
     * @param _hyperlinkText the hyperlinked text.
     * @param _hyperlinkInfo the hyperlink info.
     */
    public void printHyperlink(final String _hyperlinkText, final HyperlinkInfo _hyperlinkInfo) {
        wrapped.printHyperlink(_hyperlinkText, _hyperlinkInfo);
    }

    /**
     * Delegates the call to <code>getContentSize</code>.
     * @return the content size.
     */
    public int getContentSize() {
        return wrapped.getContentSize();
    }

    /**
     * Delegates the call to <code>canPause</code>.
     * @return the value of <code>canPause</code>.
     */
    public boolean canPause() {
        return wrapped.canPause();
    }

    /**
     * Delegates the call to <code>getComponent</code>.
     * @return the UI component.
     */
    public JComponent getComponent() {
        return wrapped.getComponent();
    }

    /**
     * Delegates the call to <code>dispose</code>.
     */
    public void dispose() {
        wrapped.dispose();
    }
}
