/*
 *    Copyright 2006 Alejandro Gomez
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 *
 */
package ar.amgomez.idea.remotetail.ui;

import ar.amgomez.idea.remotetail.LogConsoleModel;
import ar.amgomez.idea.remotetail.logging.LogEntry;
import ar.amgomez.idea.remotetail.logging.LogLevel;
import com.intellij.execution.ui.ConsoleView;
import com.intellij.execution.ui.ConsoleViewContentType;

import java.awt.Component;

/**
 * <code>LogConsoleView</code> implementation that uses a <code>ConsoleView</code> for viewing the log file.
 * @author Alejandro Gomez
 */
public class LogConsoleViewImpl extends AbstractLogConsoleView {

    /**
     * The line separator.
     */
    private static final String LINE_SEPARATOR = "\n";

    /**
     * The <code>ConsoleView</code> for viewing the log file.
     */
    private ConsoleView consoleView;

    /**
     * Constructs a new <code>LogConsoleViewImpl</code>.
     * @param _model the model associated with this
     * @param _consoleView the <code>ConsoleView</code> used for viewing the log file.
     */
    public LogConsoleViewImpl(final LogConsoleModel _model, final ConsoleView _consoleView) {
        super(_model);
        consoleView = _consoleView;
    }

    /**
     * Prints a <code>LogEntry</code>.
     * @param _logEntry the <code>LogEntry</code> to print.
     */
    @Override
    protected void printLogEntry(final LogEntry _logEntry) {
        consoleView.print(_logEntry.getText() + LINE_SEPARATOR, getContentType(_logEntry));
    }

    /**
     * Clears the console.
     */
    @Override
    protected void clear() {
        consoleView.clear();
    }

    /**
     * Returns the component for viewing the log.
     * @return the component for viewing the log.
     */
    @Override
    protected Component getConsoleComponent() {
        return consoleView.getComponent();
    }

    /**
     * Disposes this component.
     */
    public void dispose() {
        consoleView.dispose();
    }

    /**
     * Returns the <code>ConsoleViewContentType</code> associated with the given <code>LogEntry</code> according to its <code>LogLevel</code>.
     * @param _logEntry the <code>LogEntry</code>.
     * @return the <code>ConsoleViewContentType</code> associated with the given <code>LogEntry</code>.
     */
    private static ConsoleViewContentType getContentType(final LogEntry _logEntry) {
        return _logEntry.getCategory().getLevel() == LogLevel.ERROR ? ConsoleViewContentType.ERROR_OUTPUT : ConsoleViewContentType.NORMAL_OUTPUT;
    }
}
