/*
 *    Copyright 2006 Alejandro Gomez
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 *
 */
package ar.amgomez.idea.remotetail.ui;

import ar.amgomez.idea.remotetail.LogConsoleModel;
import ar.amgomez.idea.remotetail.LogConsoleModelImpl;
import ar.amgomez.idea.remotetail.MultipleConsolesModel;
import ar.amgomez.idea.remotetail.RemoteTailBundle;
import ar.amgomez.idea.remotetail.RemoteTailHelper;
import ar.amgomez.idea.remotetail.RemoteTailPlugin;
import ar.amgomez.idea.remotetail.events.MultipleConsolesAdapter;
import ar.amgomez.idea.remotetail.logging.LogEntryProvider;
import ar.amgomez.idea.remotetail.logging.LogEntryProviderFactory;
import ar.amgomez.idea.remotetail.net.ClientFactory;
import ar.amgomez.idea.remotetail.settings.ConnectionInfo;
import ar.amgomez.idea.remotetail.settings.Settings;
import ar.amgomez.idea.util.IdeaVersionsFacade;
import com.intellij.execution.ui.ConsoleView;
import com.intellij.openapi.actionSystem.ActionManager;
import com.intellij.openapi.actionSystem.AnAction;
import com.intellij.openapi.actionSystem.AnActionEvent;
import com.intellij.openapi.actionSystem.DefaultActionGroup;
import com.intellij.openapi.application.Application;
import com.intellij.openapi.application.ApplicationManager;
import com.intellij.openapi.diagnostic.Logger;
import com.intellij.openapi.progress.ProgressIndicator;
import com.intellij.openapi.progress.ProgressManager;
import com.intellij.openapi.project.Project;
import com.intellij.openapi.ui.Messages;

import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import javax.swing.SwingConstants;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import java.awt.BorderLayout;
import java.awt.Component;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

/**
 * The UI component containing many log consoles.
 * @author Alejandro Gomez
 */
public class MultipleConsolesViewImpl implements MultipleConsolesView {

    /**
     * To inform application errors using IDEA's logging mechanism.
     */
    private static final Logger LOGGER = Logger.getInstance(MultipleConsolesViewImpl.class.getName());

    /**
     * The model associated with this component.
     */
    private MultipleConsolesModel model;
    /**
     * The UI component.
     */
    private JComponent component;
    /**
     * The tabbed UI component.
     */
    private JTabbedPane tabbedPane;
    /**
     * The <code>List</code> of views contained in the tabbed pane.
     */
    private List<LogConsoleView> views;

    /**
     * Constructs a new <code>MultipleConsolesViewImpl</code>.
     * @param _model the model associated with the newly created view component.
     */
    public MultipleConsolesViewImpl(final MultipleConsolesModel _model) {
        model = _model;
        views = new ArrayList<LogConsoleView>();
        model.addMultipleConsolesListener(new MultipleConsolesAdapter() {
            @Override
            public void consoleAdded(final LogConsoleModel _logConsoleModel) {
                addTab(_logConsoleModel);
            }

            @Override
            public void consoleRemoved(final int _index) {
                final LogConsoleView view = views.remove(_index);
                view.dispose();
                tabbedPane.remove(_index);
                tabbedPane.setSelectedIndex(tabbedPane.getSelectedIndex());//FIXME ???
            }

            @Override
            public void consoleSelected(final int _index) {
                tabbedPane.setSelectedIndex(_index);
            }

            private void addTab(final LogConsoleModel _logConsoleModel) {
                final Application application = ApplicationManager.getApplication();
                if (application.isDispatchThread()) {
                    final LogConsoleView logConsoleView = new LogConsoleViewImpl(_logConsoleModel, model.getProject().getComponent(RemoteTailPlugin.class).getComponent(ConsoleView.class));
                    final Component tabComponent = logConsoleView.getComponent();
                    tabbedPane.setSelectedComponent(tabbedPane.add(logConsoleView.getTitle(), tabComponent));
                    views.add(logConsoleView);
                } else {
                    application.invokeLater(new Runnable() {
                        public void run() {
                            addTab(_logConsoleModel);
                        }
                    });
                }
            }
        });
        init();
    }

    /**
     * Initializes the UI components.
     */
    private void init() {
        component = new JPanel(new BorderLayout());
        component.add(buildToolbar(), BorderLayout.WEST);
        component.add(buildViewPanel(), BorderLayout.CENTER);
    }

    /**
     * Builds the toolbar.
     * @return a component containing all the toolbar action buttons.
     */
    private Component buildToolbar() {
        final ActionManager manager = ActionManager.getInstance();
        final DefaultActionGroup actionGroup = new DefaultActionGroup();
        actionGroup.add(new AddLogConsoleAction());
        actionGroup.add(new RemoveLogConsoleAction());
        actionGroup.addSeparator();
        actionGroup.add(manager.getAction(RemoteTailPlugin.REMOTE_TAIL_SETTINGS_ACTION_KEY));
        actionGroup.addSeparator();
        actionGroup.add(manager.getAction(RemoteTailPlugin.REMOTE_TAIL_HELP_ACTION_KEY));
        actionGroup.add(manager.getAction(RemoteTailPlugin.REMOTE_TAIL_ABOUT_ACTION_KEY));
        return manager.createActionToolbar(RemoteTailPlugin.REMOTE_TAIL_TOOLBAR_NAME, actionGroup, false).getComponent();
    }

    /**
     * Builds the view panel.
     * @return a component containing the tabbed panel.
     */
    private Component buildViewPanel() {
        tabbedPane = new JTabbedPane(SwingConstants.TOP, JTabbedPane.SCROLL_TAB_LAYOUT);
        tabbedPane.addChangeListener(new ChangeListener() {
            public void stateChanged(final ChangeEvent _event) {
                model.setSelectedLogConsole(tabbedPane.getSelectedIndex());
            }
        });
        return tabbedPane;
    }

    /**
     * Returns the UI component.
     * @return the UI component.
     */
    public JComponent getComponent() {
        return component;
    }

    /**
     * Disposes the component.
     */
    public void dispose() {
        //FIXME lifecycle not working on ConsoleViewWrapper
        for (final LogConsoleView view : views) {
            view.dispose();
        }
    }

    /**
     * The action that adds a new log console to the tabbed panel.
     * @author Alejandro Gomez
     */
    private class AddLogConsoleAction extends AnAction {

        /**
         * Constructs the action.
         */
        private AddLogConsoleAction() {
            super(RemoteTailBundle.message("remotetail.addLog.text"), RemoteTailBundle.message("remotetail.addLog.description"), RemoteTailBundle.icon("remotetail.addLog.icon"));
        }

        /**
         * The action handler.
         * @param _event information about the invocation.
         */
        @Override
        public void actionPerformed(final AnActionEvent _event) {
            final Project project = RemoteTailHelper.getProject(_event);
            final ConnectDialog connectDialog = new ConnectDialog(project, RemoteTailHelper.getSettings());
            connectDialog.show();
            if (connectDialog.isOK()) {
                IdeaVersionsFacade.runProcessWithProgressSynchronously(new Runnable() {
                    public void run() {
                        final Settings settings = RemoteTailHelper.getSettings();
                        final ConnectionInfo connectionInfo = settings.getConnectionInfo();
                        final ProgressIndicator indicator = ProgressManager.getInstance().getProgressIndicator();
                        indicator.setIndeterminate(true);
                        indicator.setText(RemoteTailBundle.message("remotetail.connection.progress.text", connectionInfo.getHost(), connectionInfo.getUsername()));
                        final LogEntryProvider logEntryProvider = LogEntryProviderFactory.getInstance().getLogEntryProvider(ClientFactory.getInstance().getClient(connectionInfo));
                        try {
                            final boolean result = logEntryProvider.connect();
                            if (result) {
                                final LogConsoleModel logConsoleModel = new LogConsoleModelImpl(logEntryProvider, settings.getBufferSize());
                                model.addLogConsole(logConsoleModel);
                            } else {
                                final String message = RemoteTailBundle.message("remotetail.connectError.message", logEntryProvider.getHost());
                                ApplicationManager.getApplication().invokeLater(new Runnable() {
                                    public void run() {
                                        Messages.showErrorDialog(message, RemoteTailBundle.message("remotetail.connectError.title"));
                                    }
                                });
                            }
                        } catch (IOException ex) {
                            LOGGER.info(ex);
                            final String message = RemoteTailBundle.message("remotetail.connectException.message", logEntryProvider.getHost(), ex.getMessage());
                            ApplicationManager.getApplication().invokeLater(new Runnable() {
                                public void run() {
                                    Messages.showErrorDialog(message, RemoteTailBundle.message("remotetail.connectException.title"));
                                }
                            });
                        }
                    }
                }, RemoteTailBundle.message("remotetail.connection.progress.title"), false, project);
            }
        }
    }

    /**
     * The action that removes the currently selected log console from the tabbed panel.
     * @author Alejandro Gomez
     */
    private class RemoveLogConsoleAction extends AnAction {

        /**
         * Constructs the action.
         */
        private RemoveLogConsoleAction() {
            super(RemoteTailBundle.message("remotetail.removeLog.text"), RemoteTailBundle.message("remotetail.removeLog.description"), RemoteTailBundle.icon("remotetail.removeLog.icon"));
        }

        /**
         * Updates the state of the action.
         * @param _event information about the invocation.
         */
        @Override
        public void update(final AnActionEvent _event) {
            super.update(_event);
            _event.getPresentation().setEnabled(model.getSelectedLogConsole() != null);
        }

        /**
         * The action handler.
         * @param _event information about the invocation.
         */
        @Override
        public void actionPerformed(final AnActionEvent _event) {
            model.removeLogConsole();
        }
    }
}
