/*
 *    Copyright 2006 Alejandro Gomez
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 *
 */
package ar.amgomez.idea.remotetail.ui;

import ar.amgomez.idea.remotetail.RemoteTailBundle;
import ar.amgomez.idea.remotetail.net.RemoteFile;
import ar.amgomez.idea.remotetail.net.RemoteFileFactory;
import ar.amgomez.idea.remotetail.net.SSHClient;
import ar.amgomez.idea.util.GenericCollectionUtils;
import ar.amgomez.idea.util.GenericPredicate;
import ar.amgomez.idea.util.GenericTransformer;
import com.intellij.openapi.diagnostic.Logger;
import com.sshtools.j2ssh.SftpClient;
import com.sshtools.j2ssh.sftp.SftpFile;
import org.apache.commons.collections.IteratorUtils;
import org.jetbrains.annotations.Nullable;

import javax.swing.Icon;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTree;
import javax.swing.ScrollPaneConstants;
import javax.swing.event.TreeExpansionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.event.TreeWillExpandListener;
import javax.swing.tree.DefaultTreeCellRenderer;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.ExpandVetoException;
import javax.swing.tree.TreeModel;
import javax.swing.tree.TreeNode;
import javax.swing.tree.TreePath;
import javax.swing.tree.TreeSelectionModel;
import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.event.KeyListener;
import java.io.IOException;
import java.util.Collections;
import java.util.Enumeration;
import java.util.List;

/**
 * Class RemoteFileChooser.
 * @author Alejandro Gomez
 */
public class RemoteFileChooser {

    /**
     * To inform application errors using IDEA's logging mechanism.
     */
    private static final Logger LOGGER = Logger.getInstance(RemoteFileChooser.class.getName());

    /**
     * The icon for folders.
     */
    private static final Icon FOLDER_ICON = RemoteTailBundle.icon("remotetail.fileBrowser.folderIcon");
    /**
     * The icon for unreadable folders.
     */
    private static final Icon FOLDER_LOCKED_ICON = RemoteTailBundle.icon("remotetail.fileBrowser.folderLockedIcon");
    /**
     * The icon for open folders.
     */
    private static final Icon FOLDER_OPEN_ICON = RemoteTailBundle.icon("remotetail.fileBrowser.folderOpenIcon");
    /**
     * The icon for files.
     */
    private static final Icon FILE_ICON = RemoteTailBundle.icon("remotetail.fileBrowser.fileIcon");
    /**
     * The icon for unreadable files.
     */
    private static final Icon FILE_LOCKED_ICON = RemoteTailBundle.icon("remotetail.fileBrowser.fileLockedIcon");

    /**
     * The panel containing the navigation tree.
     */
    private JPanel panel;
    /**
     * The tree for navigating the file system.
     */
    private JTree tree;
    /**
     * The SSH network protocol client.
     */
    private SSHClient sshClient;
    /**
     * The Secure File Transfer Protocol client.
     */
    private SftpClient sftpClient;

    /**
     * Constructs a new <code>RemoteFileChooser</code>.
     * Gets the SFTP client.
     * @param _sshClient the SSH network protocol client.
     * @throws IOException if produced while getting the SFTP client or while listing files.
     */
    public RemoteFileChooser(final SSHClient _sshClient) throws IOException {
        sshClient = _sshClient;
        sftpClient = sshClient.getSftpClient();
        panel = new JPanel(new BorderLayout());
        final SftpFile file = (SftpFile)sftpClient.ls("/").get(0);
        final RemoteFSTreeNode root = new RemoteFSTreeNode(RemoteFileFactory.getInstance().getRemoteFile(file), null);
        final TreeModel model = new DefaultTreeModel(root, true);
        tree = new JTree(model);
        tree.setCellRenderer(new RemoteFileChooserCellRenderer());
        tree.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
        tree.addTreeWillExpandListener(new TreeWillExpandListener() {
            public void treeWillExpand(final TreeExpansionEvent _event) throws ExpandVetoException {
                if (!((RemoteFSTreeNode)_event.getPath().getLastPathComponent()).getFile().canRead()) {
                    throw new ExpandVetoException(_event, "can't read");
                }
            }

            public void treeWillCollapse(final TreeExpansionEvent _event) {
            }
        });
        final JScrollPane pane = new JScrollPane(tree);
        pane.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);
        pane.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED);
        panel.add(pane);
    }

    /**
     * Returns the UI panel.
     * @return the UI panel.
     */
    public JPanel getPanel() {
        return panel;
    }

    /**
     * Returns the selected file in the tree.
     * @return the selected file.
     */
    @Nullable
    public RemoteFile getSelectedFile() {
        final TreePath selectionPath = tree.getSelectionPath();
        return selectionPath == null ? null : ((RemoteFSTreeNode)selectionPath.getLastPathComponent()).getFile();
    }

    /**
     * Returns the absolute path of the selected file.
     * @return the absolute path of the selected file.
     */
    @Nullable
    public String getSelectedFilePath() {
        final RemoteFile file = getSelectedFile();
        return file == null ? null : file.getAbsolutePath();
    }

    /**
     * Registers a new <code>TreeSelectionListener</code>.
     * @param _listener the <code>TreeSelectionListener</code> to register.
     */
    public void addTreeSelectionListener(final TreeSelectionListener _listener) {
        tree.addTreeSelectionListener(_listener);
    }

    /**
     * Unregisters a <code>TreeSelectionListener</code>.
     * @param _listener the <code>TreeSelectionListener</code> to unregister.
     */
    public void removeTreeSelectionListener(final TreeSelectionListener _listener) {
        tree.removeTreeSelectionListener(_listener);
    }

    /**
     * Registers a new <code>KeyListener</code>.
     * @param _listener the <code>KeyListener</code> to register.
     */
    public void addKeyListener(final KeyListener _listener) {
        tree.addKeyListener(_listener);
    }

    /**
     * Unregisters a <code>KeyListener</code>.
     * @param _listener the <code>KeyListener</code> to unregister.
     */
    public void removeKeyListener(final KeyListener _listener) {
        tree.removeKeyListener(_listener);
    }

    /**
     * A tree node containing a <code>RemoteFile</code>.
     * @author Alejandro Gomez
     */
    class RemoteFSTreeNode implements TreeNode {

        /**
         * The <code>RemoteFile</code>.
         */
        private RemoteFile file;
        /**
         * The parent node.
         */
        private RemoteFSTreeNode parent;
        /**
         * The <code>List</code> of children nodes.
         */
        private List<TreeNode> children;

        /**
         * Constructs a new <code>RemoteFSTreeNode</code>.
         * @param _file the <code>RemoteFile</code> associated with this node.
         * @param _parent the parent node.
         */
        RemoteFSTreeNode(final RemoteFile _file, final RemoteFSTreeNode _parent) {
            file = _file;
            parent = _parent;
        }

        /**
         * Returns the child <code>TreeNode</code> at index
         * <code>childIndex</code>.
         */
        public TreeNode getChildAt(final int _childIndex) {
            return getChildren().get(_childIndex);
        }

        /**
         * Returns the number of children <code>TreeNode</code> this node contains.
         */
        public int getChildCount() {
            return getChildren().size();
        }

        /**
         * Returns the parent <code>TreeNode</code> of this node.
         */
        public TreeNode getParent() {
            return parent;
        }

        /**
         * Returns the index of the passed node in this node's children.
         * @return this index of <code>_node</code> or -1 if not present.
         */
        public int getIndex(final TreeNode _node) {
            return getChildren().indexOf(_node);
        }

        /**
         * Returns true if the receiver allows children.
         * @return true if the receiver allows children.
         */
        public boolean getAllowsChildren() {
            return file.isDirectory();
        }

        /**
         * Returns true if the receiver is a leaf.
         * @return true if the receiver is a leaf.
         */
        public boolean isLeaf() {
            return !file.isDirectory();
        }

        /**
         * Returns the children of this node as an <code>Enumeration</code>.
         * @return the children of this node as an <code>Enumeration</code>.
         */
        public Enumeration children() {
            return IteratorUtils.asEnumeration(getChildren().iterator());
        }

        /**
         * Returns the file associated with this node.
         * @return the file associated with this node.
         */
        public RemoteFile getFile() {
            return file;
        }

        /**
         * Returns a <code>List</code> with this node's children.
         * The <code>List</code> is ordered alphabetically with the directories first.
         * @return this node's children.
         */
        private List<TreeNode> getChildren() {
            if (children == null) {
                try {
                    final List<RemoteFile> files = RemoteFileFactory.getInstance().getRemoteFiles(sftpClient.ls(file.getAbsolutePath()));
                    GenericCollectionUtils.filter(files, new GenericPredicate<RemoteFile>() {
                        public boolean evaluate(final RemoteFile _object) {
                            return !_object.isPointer();
                        }
                    });
                    Collections.sort(files);
                    children = GenericCollectionUtils.collect(files, new GenericTransformer<TreeNode, RemoteFile>() {
                        public TreeNode transform(final RemoteFile _object) {
                            return new RemoteFSTreeNode(_object, RemoteFSTreeNode.this);
                        }
                    });
                } catch (IOException ex) {
                    if ("Permission denied".equals(ex.getMessage())) {
                        file.setCanRead(false);
                    } else {
                        LOGGER.error("error listing " + file.getAbsolutePath(), ex);
                    }
                    children = Collections.emptyList();
                }
            }
            return children;
        }
    }

    /**
     * The <code>TreeCellRenderer</code> implementation for presenting <code>RemoteFSTreeNode</code>.
     * @author Alejandro Gomez
     */
    private class RemoteFileChooserCellRenderer extends DefaultTreeCellRenderer {

        /**
         * Builds a <code>JLabel</code> with the file name and an icon representing its type and state.
         * @return a <code>JLabel</code> with the file name and an icon representing its type and state.
         */
        @Override
        public Component getTreeCellRendererComponent(final JTree _tree, final Object _value, final boolean _selected, final boolean _expanded, final boolean _leaf, final int _row, final boolean _hasFocus) {
            final Component component = super.getTreeCellRendererComponent(_tree, _value, _selected, _expanded, _leaf, _row, _hasFocus);
            final RemoteFile file = ((RemoteFSTreeNode)_value).getFile();
            setText(file.getFilename());
            if (file.isDirectory()) {
                if (file.canRead()) {
                    setIcon(_expanded ? FOLDER_OPEN_ICON : FOLDER_ICON);
                } else {
                    setIcon(FOLDER_LOCKED_ICON);
                }
            } else if (file.canRead()) {
                setIcon(FILE_ICON);
            } else {
                setIcon(FILE_LOCKED_ICON);
            }
            return component;
        }
    }
}
