/*
 *    Copyright 2006 Alejandro Gomez
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 *
 */
package ar.amgomez.idea.util;

import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;

/**
 * A version of <code>CollectionUtils</code> using Java 5 generics.
 * @author Alejandro Gomez
 */
public class GenericCollectionUtils {

    /**
     * Searches a collection for an element that satifies a predicate.
     * @param _collection the collection.
     * @param _predicate the predicate to evaluate.
     * @return true if an element of the collection satifies the predicate.
     */
    public static <T> boolean exists(@NotNull final Collection<T> _collection, @NotNull final GenericPredicate<T> _predicate) {
        return find(_collection, _predicate) != null;
    }

    /**
     * Searches a collection for an element that satifies a predicate.
     * @param _collection the collection.
     * @param _predicate the predicate to evaluate.
     * @return the element that satifies the predicate, null if none satifies it.
     */
    @Nullable
    public static <T> T find(@NotNull final Collection<T> _collection, @NotNull final GenericPredicate<T> _predicate) {
        for (final T t : _collection) {
            if (_predicate.evaluate(t)) {
                return t;
            }
        }
        return null;
    }

    /**
     * Searches a list for the first element that satifies a predicate.
     * @param _list the list.
     * @param _predicate the predicate to evaluate.
     * @return the index of the first element that satifies the predicate, -1 if none satifies it.
     */
    public static <T> int indexOf(@NotNull final List<T> _list, @NotNull final GenericPredicate<T> _predicate) {
        for (int i = 0; i < _list.size(); i++) {
            final T t = _list.get(i);
            if (_predicate.evaluate(t)) {
                return i;
            }
        }
        return -1;
    }

    /**
     * Searches a collection for all the elements that satisfy a predicate.
     * @param _collection the collection.
     * @param _predicate the predicate to evaluate.
     * @return a new collection with all the elements that satifies the predicate.
     */
    @NotNull
    public static <T> Collection<T> select(@NotNull final Collection<T> _collection, @NotNull final GenericPredicate<T> _predicate) {
        final Collection<T> result = new ArrayList<T>(_collection.size());
        for (final T t : _collection) {
            if (_predicate.evaluate(t)) {
                result.add(t);
            }
        }
        return result;
    }

    /**
     * Searches a list for all the elements that satisfy a predicate.
     * @param _list the list.
     * @param _predicate the predicate to evaluate.
     * @return a new list with all the elements that satifies the predicate.
     */
    @NotNull
    public static <T> List<T> select(@NotNull final List<T> _list, @NotNull final GenericPredicate<T> _predicate) {
        final List<T> result = new ArrayList<T>(_list.size());
        for (final T t : _list) {
            if (_predicate.evaluate(t)) {
                result.add(t);
            }
        }
        return result;
    }

    /**
     * Returns a new <code>List</code> with the original <code>List</code> elements transformed.
     * @param _list the original <code>List</code>.
     * @param _transformer the <code>GenericTransformer</code> to apply to the elements.
     * @return a new <code>List</code> with the original <code>List</code> elements transformed.
     */
    public static <R, T> List<R> collect(final List<T> _list, final GenericTransformer<R, T> _transformer) {
        final List<R> result = new ArrayList<R>(_list.size());
        for (final T t : _list) {
            result.add(_transformer.transform(t));
        }
        return result;
    }

    /**
     * Filters a <code>List</code> removing the elements not satisfying the <code>GenericPredicate</code>.
     * @param _list the original <code>List</code>.
     * @param _predicate the <code>GenericPredicate</code> for filtering the elements.
     */
    public static <T> void filter(final List<T> _list, final GenericPredicate<T> _predicate) {
        for (Iterator<T> iterator = _list.listIterator(); iterator.hasNext();) {
            if (!_predicate.evaluate(iterator.next())) {
                iterator.remove();
            }
        }
    }
}
