/*
 *    Copyright 2006 Alejandro Gomez
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 *
 */
package ar.amgomez.idea.util;

import com.intellij.execution.filters.TextConsoleBuidlerFactory;
import com.intellij.execution.filters.TextConsoleBuilder;
import com.intellij.execution.ui.ConsoleView;
import com.intellij.openapi.application.Application;
import com.intellij.openapi.application.ApplicationInfo;
import com.intellij.openapi.application.ApplicationManager;
import com.intellij.openapi.diagnostic.Logger;
import com.intellij.openapi.progress.ProgressManager;
import com.intellij.openapi.project.Project;
import org.jetbrains.annotations.NonNls;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

/**
 * Provides a common implementation for methods which differ from IDEA 5.1.2 to IDEA 6.0.
 * @author Alejandro Gomez
 */
public class IdeaVersionsFacade {

    /**
     * To inform application errors using IDEA's logging mechanism.
     */
    private static final Logger LOGGER = Logger.getInstance(IdeaVersionsFacade.class.getName());

    /**
     * The name of the method to run a process with a progress bar.
     * The name is the same for both IDEA 5 and 6.
     */
    @NonNls
    private static final String RUN_PROCESS_METHOD_NAME = "runProcessWithProgressSynchronously";

    /**
     * Creates and returns a new <code>ConsoleView</code>.
     * @param _project the active project.
     * @return a new <code>ConsoleView</code>.
     * @see TextConsoleBuilder#getConsole()
     */
    public static ConsoleView getConsoleView(final Project _project) {
        // this class was deprecated in IDEA 6 probably due to the error in its name.
        // I'm using the deprecated version for backward compatibility while it is there.
        return TextConsoleBuidlerFactory.getInstance().createBuilder(_project).getConsole();
    }

    /**
     * Runs the specified operation in a background thread and shows a modal progress dialog in the
     * main thread while the operation is executing.
     * This method was in <code>Application</code> in IDEA 5, and was moved to <code>ProgressManager</code>
     * in IDEA 6.
     * @param _process     the operation to execute.
     * @param _title       the title of the progress window.
     * @param _cancelable  whether "Cancel" button is shown on the progress window.
     * @param _project     the project in the context of which the operation is executed.
     * @return true        if the operation completed successfully, false if it was cancelled.
     * @see ProgressManager#runProcessWithProgressSynchronously(Runnable,String,boolean,Project)
     * @see com.intellij.openapi.application.Application#runProcessWithProgressSynchronously(Runnable,String,boolean,Project)
     */
    public static boolean runProcessWithProgressSynchronously(final Runnable _process, final String _title, final boolean _cancelable, final Project _project) {
        final Class clazz;
        final Object instance;
        if (isIdea5()) {
            clazz = Application.class;
            instance = ApplicationManager.getApplication();
        } else {
            clazz = ProgressManager.class;
            instance = ProgressManager.getInstance();
        }
        try {
            final Method method = clazz.getMethod(RUN_PROCESS_METHOD_NAME, Runnable.class, String.class, Boolean.TYPE, Project.class);
            return (Boolean)method.invoke(instance, _process, _title, _cancelable, _project);
        } catch (IllegalAccessException ex) {
            LOGGER.error(ex);
        } catch (InvocationTargetException ex) {
            LOGGER.error(ex);
        } catch (NoSuchMethodException ex) {
            LOGGER.error(ex);
        }
        return false;
    }

    /**
     * Checks if we are running on IDEA 6.
     * @return true if IDEA's major version number is 6.
     */
    private static boolean isIdea6() {
        return equalsMajorVersion("6");
    }

    /**
     * Checks if we are running on IDEA 5.
     * @return true if IDEA's major version number is 5.
     */
    private static boolean isIdea5() {
        return equalsMajorVersion("5");
    }

    /**
     * Checks IDEA's major version number for equality.
     * @param _majorVersion the number to compare IDEA's major version to.
     * @return true if IDEA's major version number is equal to the argument.
     */
    private static boolean equalsMajorVersion(@NonNls final String _majorVersion) {
        return _majorVersion.equals(ApplicationInfo.getInstance().getMajorVersion());
    }
}
