/*
 *    Copyright 2006 Alejandro Gomez
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 *
 */
package ar.amgomez.idea.util;

import java.util.AbstractList;
import java.util.ArrayList;
import java.util.List;
import java.util.RandomAccess;

/**
 * An implementation of <code>List</code> with a maximum capacity.
 * When the capacity is reached, new elements are added removing
 * the least recently added ones.
 * @author Alejandro Gomez
 */
public class LRAList<T> extends AbstractList<T> implements RandomAccess {

    /**
     * The default maximum capacity.
     */
    private static final int SIZE = 1000;

    /**
     * The list capacity.
     */
    private int capacity;
    /**
     * The decoratee list.
     */
    private List<T> decoratee;

    /**
     * Creates a new <code>LRAList</code> using the default maximum capacity.
     */
    public LRAList() {
        this(SIZE);
    }

    /**
     * Creates a new <code>LRAList</code> using the passed maximum capacity.
     * @param _capacity the list maximum capacity.
     */
    public LRAList(final int _capacity) {
        capacity = _capacity;
        decoratee = new ArrayList<T>(capacity);
    }

    /**
     * Returns the number of elements in this list.
     * @return the number of elements in this list.
     */
    @Override
    public int size() {
        return decoratee.size();
    }

    /**
     * Returns the element at the specified position in this list.
     * @param _index index of element to return.
     * @return the element at the specified position in this list.
     * @throws IndexOutOfBoundsException if the index is out of range (index &lt; 0 || index &gt;= size()).
     */
    @Override
    public T get(final int _index) {
        return decoratee.get(_index);
    }

    /**
     * Appends the specified element to the end of this list.
     * If the list is completed removes the least recently added
     * element to not to exceed the maximum capacity.
     * @param _element element to be appended to this list.
     * @return <tt>true</tt> (as per the general contract of the <tt>Collection.add</tt> method).
     * @throws IllegalArgumentException if some aspect of this element
     *            prevents it from being added to this list.
     */
    @Override
    public boolean add(final T _element) {
        checkCapacity();
        return decoratee.add(_element);
    }

    /**
     * Removes the element at the specified position in this list.
     * Shifts any subsequent elements to the left (subtracts one
     * from their indices).
     * Returns the element that was removed from the list.
     * @param _index the index of the element to removed.
     * @return the element previously at the specified position.
     * @throws IndexOutOfBoundsException if the index is out of range (index &lt; 0 || index &gt;= size()).
     */
    @Override
    public T remove(final int _index) {
        return decoratee.remove(_index);
    }

    /**
     * Checks if the list is full.
     * If the maximum capacity was reached removes the least recently added
     * element to make room for adding a new element.
     */
    private void checkCapacity() {
        if (decoratee.size() == capacity) {
            decoratee.remove(0);
        }
    }
}
