/*
 *    Copyright 2006 Alejandro Gomez
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 *
 */
package ar.amgomez.idea.remotetail.logging;

import junit.framework.TestCase;
import org.jetbrains.annotations.NonNls;

import java.text.Format;
import java.text.MessageFormat;

/**
 * Class LogCategoryTest.
 * @author Alejandro Gomez
 */
public class LogCategoryTest extends TestCase {

    /**
     * Field MESSAGE_PATTERN.
     */
    @NonNls
    private static final String MESSAGE_PATTERN = "qwerty {0} asdfg";
    /**
     * Field MESSAGE_FORMAT.
     */
    private static final Format MESSAGE_FORMAT = new MessageFormat(MESSAGE_PATTERN);
    /**
     * Field UNMATCHED_LINE.
     */
    @NonNls
    private static final String UNMATCHED_LINE = "zxcvb";

    /**
     * Method testUnknownGetLevel.
     */
    public void testUnknownGetLevel() {
        testGetLevel(LogCategory.UNKNOWN, null);
    }

    /**
     * Method testInfoGetLevel.
     */
    public void testInfoGetLevel() {
        testGetLevel(LogCategory.INFO, LogLevel.INFO);
    }

    /**
     * Method testDebugGetLevel.
     */
    public void testDebugGetLevel() {
        testGetLevel(LogCategory.DEBUG, LogLevel.INFO);
    }

    /**
     * Method testWarningGetLevel.
     */
    public void testWarningGetLevel() {
        testGetLevel(LogCategory.WARNING, LogLevel.WARNING);
    }

    /**
     * Method testErrorGetLevel.
     */
    public void testErrorGetLevel() {
        testGetLevel(LogCategory.ERROR, LogLevel.ERROR);
    }

    /**
     * Method testSevereGetLevel.
     */
    public void testSevereGetLevel() {
        testGetLevel(LogCategory.SEVERE, LogLevel.ERROR);
    }

    /**
     * Method testFatalGetLevel.
     */
    public void testFatalGetLevel() {
        testGetLevel(LogCategory.FATAL, LogLevel.ERROR);
    }

    /**
     * Method testGetLevel.
     * @param _category
     * @param _expectedLevel
     */
    private static void testGetLevel(final LogCategory _category, final LogLevel _expectedLevel) {
        assertEquals(_expectedLevel, _category.getLevel());
    }

    /**
     * Method testInfoMatches.
     */
    public void testInfoMatches() {
        testMatches("INFO", LogCategory.INFO);
    }

    /**
     * Method testDebugMatches.
     */
    public void testDebugMatches() {
        testMatches("DEBUG", LogCategory.DEBUG);
    }

    /**
     * Method testWarningMatches.
     */
    public void testWarningMatches() {
        testMatches("WARN", LogCategory.WARNING);
        testMatches("WARNING", LogCategory.WARNING);
    }

    /**
     * Method testErrorMatches.
     */
    public void testErrorMatches() {
        testMatches("ERROR", LogCategory.ERROR);
    }

    /**
     * Method testSevereMatches.
     */
    public void testSevereMatches() {
        testMatches("SEVERE", LogCategory.SEVERE);
    }

    /**
     * Method testFatalMatches.
     */
    public void testFatalMatches() {
        testMatches("FATAL", LogCategory.FATAL);
    }

    /**
     * Method testMatches.
     * @param _word
     * @param _category
     */
    private static void testMatches(@NonNls final String _word, final LogCategory _category) {
        assertTrue(_category.matches(MESSAGE_FORMAT.format(new Object[]{_word})));
    }

    /**
     * Method testUnknownDoesntMatch.
     */
    public void testUnknownDoesntMatch() {
        testDoesntMatch(LogCategory.UNKNOWN);
    }

    /**
     * Method testInfoDoesntMatch.
     */
    public void testInfoDoesntMatch() {
        testDoesntMatch(LogCategory.INFO);
    }

    /**
     * Method testDebugDoesntMatch.
     */
    public void testDebugDoesntMatch() {
        testDoesntMatch(LogCategory.DEBUG);
    }

    /**
     * Method testWarningDoesntMatch.
     */
    public void testWarningDoesntMatch() {
        testDoesntMatch(LogCategory.WARNING);
    }

    /**
     * Method testErrorDoesntMatch.
     */
    public void testErrorDoesntMatch() {
        testDoesntMatch(LogCategory.ERROR);
    }

    /**
     * Method testSevereDoesntMatch.
     */
    public void testSevereDoesntMatch() {
        testDoesntMatch(LogCategory.SEVERE);
    }

    /**
     * Method testFatalDoesntMatch.
     */
    public void testFatalDoesntMatch() {
        testDoesntMatch(LogCategory.FATAL);
    }

    /**
     * Method testDoesntMatch.
     * @param _category
     */
    private static void testDoesntMatch(final LogCategory _category) {
        assertFalse(_category.matches(UNMATCHED_LINE));
    }

    /**
     * Method testInfoIsLevelGreaterOrEqual.
     */
    public void testInfoIsLevelGreaterOrEqual() {
        assertTrue(LogCategory.INFO.isLevelGreaterOrEqual(LogLevel.INFO));
        assertFalse(LogCategory.INFO.isLevelGreaterOrEqual(LogLevel.WARNING));
        assertFalse(LogCategory.INFO.isLevelGreaterOrEqual(LogLevel.ERROR));
    }

    /**
     * Method testDebugIsLevelGreaterOrEqual.
     */
    public void testDebugIsLevelGreaterOrEqual() {
        assertTrue(LogCategory.DEBUG.isLevelGreaterOrEqual(LogLevel.INFO));
        assertFalse(LogCategory.DEBUG.isLevelGreaterOrEqual(LogLevel.WARNING));
        assertFalse(LogCategory.DEBUG.isLevelGreaterOrEqual(LogLevel.ERROR));
    }

    /**
     * Method testWarningIsLevelGreaterOrEqual.
     */
    public void testWarningIsLevelGreaterOrEqual() {
        assertTrue(LogCategory.WARNING.isLevelGreaterOrEqual(LogLevel.INFO));
        assertTrue(LogCategory.WARNING.isLevelGreaterOrEqual(LogLevel.WARNING));
        assertFalse(LogCategory.WARNING.isLevelGreaterOrEqual(LogLevel.ERROR));
    }

    /**
     * Method testErrorIsLevelGreaterOrEqual.
     */
    public void testErrorIsLevelGreaterOrEqual() {
        assertTrue(LogCategory.ERROR.isLevelGreaterOrEqual(LogLevel.INFO));
        assertTrue(LogCategory.ERROR.isLevelGreaterOrEqual(LogLevel.WARNING));
        assertTrue(LogCategory.ERROR.isLevelGreaterOrEqual(LogLevel.ERROR));
    }

    /**
     * Method testSevereIsLevelGreaterOrEqual.
     */
    public void testSevereIsLevelGreaterOrEqual() {
        assertTrue(LogCategory.SEVERE.isLevelGreaterOrEqual(LogLevel.INFO));
        assertTrue(LogCategory.SEVERE.isLevelGreaterOrEqual(LogLevel.WARNING));
        assertTrue(LogCategory.SEVERE.isLevelGreaterOrEqual(LogLevel.ERROR));
    }

    /**
     * Method testFatalIsLevelGreaterOrEqual.
     */
    public void testFatalIsLevelGreaterOrEqual() {
        assertTrue(LogCategory.FATAL.isLevelGreaterOrEqual(LogLevel.INFO));
        assertTrue(LogCategory.FATAL.isLevelGreaterOrEqual(LogLevel.WARNING));
        assertTrue(LogCategory.FATAL.isLevelGreaterOrEqual(LogLevel.ERROR));
    }
}
